@@ -1,4 +1,4 @@
-{-# LANGUAGE CPP, TupleSections, ScopedTypeVariables #-}
+{-# LANGUAGE CPP, TupleSections, ScopedTypeVariables, PatternGuards #-}
 {-
 Copyright (C) 2006-2016 John MacFarlane <jgm@berkeley.edu>
 
@@ -42,14 +42,15 @@ import Text.Pandoc.MediaBag ( mediaDirectory, extractMediaBag, MediaBag )
 import Text.Pandoc.XML ( toEntities )
 import Text.Pandoc.SelfContained ( makeSelfContained )
 import Text.Pandoc.Process (pipeProcess)
-import Text.Highlighting.Kate ( languages, Style, tango, pygments,
-         espresso, zenburn, kate, haddock, monochrome )
-import System.Environment ( getArgs, getProgName )
+import Skylighting ( defaultSyntaxMap, Syntax(..), Style, tango, pygments,
+         espresso, zenburn, kate, haddock, breezeDark, monochrome )
+import System.Environment ( getArgs, getProgName, getEnvironment )
 import System.Exit ( ExitCode (..), exitSuccess )
 import System.FilePath
 import System.Console.GetOpt
+import qualified Data.Set as Set
 import Data.Char ( toLower, toUpper )
-import Data.List ( delete, intercalate, isPrefixOf, isSuffixOf, sort )
+import Data.List ( intercalate, isPrefixOf, isSuffixOf, sort )
 import System.Directory ( getAppUserDataDirectory, findExecutable,
                           doesFileExist, Permissions(..), getPermissions )
 import System.IO ( stdout, stderr )
@@ -72,7 +73,10 @@ import Control.Applicative ((<|>))
 import Text.Pandoc.Readers.Txt2Tags (getT2TMeta)
 import Paths_pandoc (getDataDir)
 import Text.Printf (printf)
-import Text.Pandoc.Error
+#ifndef _WINDOWS
+import System.Posix.Terminal (queryTerminal)
+import System.Posix.IO (stdOutput)
+#endif
 
 type Transform = Pandoc -> Pandoc
 
@@ -87,11 +91,8 @@ copyrightMessage = intercalate "\n" [
 
 compileInfo :: String
 compileInfo =
-  "\nCompiled with texmath " ++
-  VERSION_texmath ++ ", highlighting-kate " ++ VERSION_highlighting_kate ++
-   ".\nSyntax highlighting is supported for the following languages:\n    " ++
-       wrapWords 4 78
-       [map toLower l | l <- languages, l /= "Alert" && l /= "Alert_indent"]
+  "\nCompiled with pandoc-types " ++ VERSION_pandoc_types ++ ", texmath " ++
+  VERSION_texmath ++ ", skylighting " ++ VERSION_skylighting
 
 -- | Converts a list of strings into a single string with the items printed as
 -- comma separated words in lines with a maximum line length.
@@ -114,49 +115,50 @@ isTextFormat s = s `notElem` ["odt","docx","epub","epub3"]
 
 externalFilter :: FilePath -> [String] -> Pandoc -> IO Pandoc
 externalFilter f args' d = do
-      mbexe <- if '/' `elem` f
-                  -- don't check PATH if filter name has a path
-                  then return Nothing
-                  -- we catch isDoesNotExistError because this will
-                  -- be triggered if PATH not set:
-                  else E.catch (findExecutable f)
-                                 (\e -> if isDoesNotExistError e
-                                           then return Nothing
-                                           else throwIO e)
-      (f', args'') <- case mbexe of
-                           Just x  -> return (x, args')
-                           Nothing -> do
-                             exists <- doesFileExist f
-                             if exists
-                                then do
-                                  isExecutable <- executable `fmap`
-                                                    getPermissions f
-                                  return $
-                                    case map toLower $ takeExtension f of
-                                         _ | isExecutable -> (f, args')
-                                         ".py"  -> ("python", f:args')
-                                         ".hs"  -> ("runhaskell", f:args')
-                                         ".pl"  -> ("perl", f:args')
-                                         ".rb"  -> ("ruby", f:args')
-                                         ".php" -> ("php", f:args')
-                                         _      -> (f, args')
-                                else err 85 $ "Filter " ++ f ++ " not found"
-      when (f' /= f) $ do
-          mbExe <- findExecutable f'
-          when (isNothing mbExe) $
-            err 83 $ "Error running filter " ++ f ++ "\n" ++
-                      show f' ++ " not found in path."
-      (exitcode, outbs, errbs) <- E.handle filterException $
-                                    pipeProcess Nothing f' args'' $ encode d
-      unless (B.null errbs) $ B.hPutStr stderr errbs
-      case exitcode of
-           ExitSuccess    -> return $ either error id $ eitherDecode' outbs
-           ExitFailure ec -> err 83 $ "Error running filter " ++ f ++ "\n" ++
-                                       "Filter returned error status " ++ show ec
+  exists <- doesFileExist f
+  isExecutable <- if exists
+                     then executable <$> getPermissions f
+                     else return True
+  let (f', args'') = if exists
+                        then case map toLower (takeExtension f) of
+                                  _ | isExecutable -> ("." </> f, args')
+                                  ".py"  -> ("python", f:args')
+                                  ".hs"  -> ("runhaskell", f:args')
+                                  ".pl"  -> ("perl", f:args')
+                                  ".rb"  -> ("ruby", f:args')
+                                  ".php" -> ("php", f:args')
+                                  ".js"  -> ("node", f:args')
+                                  _      -> (f, args')
+                        else (f, args')
+  unless (exists && isExecutable) $ do
+    mbExe <- findExecutable f'
+    when (isNothing mbExe) $
+      err 83 $ "Error running filter " ++ f ++  ":\n" ++
+               "Could not find executable '" ++ f' ++ "'."
+  env <- getEnvironment
+  let env' = Just $ ("PANDOC_VERSION", pandocVersion) : env
+  (exitcode, outbs, errbs) <- E.handle filterException $
+                              pipeProcess env' f' args'' $ encode d
+  unless (B.null errbs) $ B.hPutStr stderr errbs
+  case exitcode of
+       ExitSuccess    -> return $ either error id $ eitherDecode' outbs
+       ExitFailure ec -> err 83 $ "Error running filter " ++ f ++ "\n" ++
+                                  "Filter returned error status " ++ show ec
  where filterException :: E.SomeException -> IO a
        filterException e = err 83 $ "Error running filter " ++ f ++ "\n" ++
                                        show e
 
+highlightingStyles :: [(String, Style)]
+highlightingStyles =
+  [("pygments", pygments),
+   ("tango", tango),
+   ("espresso", espresso),
+   ("zenburn", zenburn),
+   ("kate", kate),
+   ("monochrome", monochrome),
+   ("breezedark", breezeDark),
+   ("haddock", haddock)]
+
 -- | Data structure for command line options.
 data Opt = Opt
     { optTabStop           :: Int     -- ^ Number of spaces per tab
@@ -182,7 +184,7 @@ data Opt = Opt
     , optHtmlQTags         :: Bool    -- ^ Use <q> tags in HTML
     , optHighlight         :: Bool    -- ^ Highlight source code
     , optHighlightStyle    :: Style   -- ^ Style to use for highlighted code
-    , optChapters          :: Bool    -- ^ Use chapter for top-level sects
+    , optTopLevelDivision  :: TopLevelDivision -- ^ Type of the top-level divisions
     , optHTMLMathMethod    :: HTMLMathMethod -- ^ Method to print HTML math
     , optReferenceODT      :: Maybe FilePath -- ^ Path of reference.odt
     , optReferenceDocx     :: Maybe FilePath -- ^ Path of reference.docx
@@ -195,6 +197,7 @@ data Opt = Opt
     , optIgnoreArgs        :: Bool    -- ^ Ignore command-line arguments
     , optVerbose           :: Bool    -- ^ Verbose diagnostic output
     , optReferenceLinks    :: Bool    -- ^ Use reference links in writing markdown, rst
+    , optReferenceLocation :: ReferenceLocation -- ^ location for footnotes and link references in markdown output
     , optDpi               :: Int     -- ^ Dpi
     , optWrapText          :: WrapOption  -- ^ Options for wrapping text
     , optColumns           :: Int     -- ^ Line length in characters
@@ -246,7 +249,7 @@ defaultOpts = Opt
     , optHtmlQTags             = False
     , optHighlight             = True
     , optHighlightStyle        = pygments
-    , optChapters              = False
+    , optTopLevelDivision      = TopLevelDefault
     , optHTMLMathMethod        = PlainMath
     , optReferenceODT          = Nothing
     , optReferenceDocx         = Nothing
@@ -259,6 +262,7 @@ defaultOpts = Opt
     , optIgnoreArgs            = False
     , optVerbose               = False
     , optReferenceLinks        = False
+    , optReferenceLocation     = EndOfDocument
     , optDpi                   = 96
     , optWrapText              = WrapAuto
     , optColumns               = 72
@@ -477,7 +481,7 @@ options =
                     case safeRead ("Wrap" ++ uppercaseFirstLetter arg) of
                           Just o   -> return opt { optWrapText = o }
                           Nothing  -> err 77 "--wrap must be auto, none, or preserve")
-                 "[auto|none|preserve]")
+                 "auto|none|preserve")
                  "" -- "Option for wrapping text in output"
 
     , Option "" ["columns"]
@@ -514,17 +518,9 @@ options =
     , Option "" ["highlight-style"]
                 (ReqArg
                  (\arg opt -> do
-                   newStyle <- case map toLower arg of
-                                     "pygments"   -> return pygments
-                                     "tango"      -> return tango
-                                     "espresso"   -> return espresso
-                                     "zenburn"    -> return zenburn
-                                     "kate"       -> return kate
-                                     "monochrome" -> return monochrome
-                                     "haddock"    -> return haddock
-                                     _            -> err 39 $
-                                         "Unknown style :" ++ arg
-                   return opt{ optHighlightStyle = newStyle })
+                   case lookup (map toLower arg) highlightingStyles of
+                         Just s -> return opt{ optHighlightStyle = s }
+                         Nothing -> err 39 $ "Unknown style: " ++ arg)
                  "STYLE")
                  "" -- "Style for highlighted code"
 
@@ -583,6 +579,19 @@ options =
                   (\opt -> return opt { optReferenceLinks = True } ))
                  "" -- "Use reference links in parsing HTML"
 
+    , Option "" ["reference-location"]
+                 (ReqArg
+                  (\arg opt -> do
+                     action <- case arg of
+                            "block"    -> return EndOfBlock
+                            "section"  -> return EndOfSection
+                            "document" -> return EndOfDocument
+                            _        -> err 6
+                               ("Unknown option for reference-location: " ++ arg)
+                     return opt { optReferenceLocation = action })
+                  "block|section|document")
+                 "" -- "Accepting or reject MS Word track-changes.""
+
     , Option "" ["atx-headers"]
                  (NoArg
                   (\opt -> return opt { optSetextHeaders = False } ))
@@ -590,9 +599,22 @@ options =
 
     , Option "" ["chapters"]
                  (NoArg
-                  (\opt -> return opt { optChapters = True }))
+                  (\opt -> do warn $ "--chapters is deprecated. " ++
+                                     "Use --top-level-division=chapter instead."
+                              return opt { optTopLevelDivision = TopLevelChapter }))
                  "" -- "Use chapter for top-level sections in LaTeX, DocBook"
 
+    , Option "" ["top-level-division"]
+                 (ReqArg
+                  (\arg opt -> do
+                      let tldName = "TopLevel" ++ uppercaseFirstLetter arg
+                      case safeRead tldName of
+                        Just tlDiv -> return opt { optTopLevelDivision = tlDiv }
+                        _       -> err 76 ("Top-level division must be " ++
+                                           "section,  chapter, part, or default"))
+                   "section|chapter|part")
+                 "" -- "Use top-level division type in LaTeX, ConTeXt, DocBook"
+
     , Option "N" ["number-sections"]
                  (NoArg
                   (\opt -> return opt { optNumberSections = True }))
@@ -822,7 +844,7 @@ options =
     , Option "" ["webtex"]
                  (OptArg
                   (\arg opt -> do
-                      let url' = fromMaybe "http://chart.apis.google.com/chart?cht=tx&chl=" arg
+                      let url' = fromMaybe "https://latex.codecogs.com/png.latex?" arg
                       return opt { optHTMLMathMethod = WebTeX url' })
                   "URL")
                  "" -- "Use web service for HTML math"
@@ -845,7 +867,7 @@ options =
                   (\arg opt ->
                       return opt
                         { optKaTeXJS =
-                           arg <|> Just "https://cdnjs.cloudflare.com/ajax/libs/KaTeX/0.5.1/katex.min.js"})
+                           arg <|> Just "https://cdnjs.cloudflare.com/ajax/libs/KaTeX/0.6.0/katex.min.js"})
                   "URL")
                   "" -- Use KaTeX for HTML Math
 
@@ -892,16 +914,68 @@ options =
                      let allopts = unwords (concatMap optnames options)
                      UTF8.hPutStrLn stdout $ printf tpl allopts
                          (unwords (map fst readers))
-                         (unwords ("pdf": map fst writers))
+                         (unwords (map fst writers))
+                         (unwords $ map fst highlightingStyles)
                          ddir
                      exitSuccess ))
                  "" -- "Print bash completion script"
 
+    , Option "" ["list-input-formats"]
+                 (NoArg
+                  (\_ -> do
+                     let readers'names = sort (map fst readers)
+                     mapM_ (UTF8.hPutStrLn stdout) readers'names
+                     exitSuccess ))
+                 ""
+
+    , Option "" ["list-output-formats"]
+                 (NoArg
+                  (\_ -> do
+                     let writers'names = sort (map fst writers)
+                     mapM_ (UTF8.hPutStrLn stdout) writers'names
+                     exitSuccess ))
+                 ""
+
+    , Option "" ["list-extensions"]
+                 (NoArg
+                  (\_ -> do
+                     let showExt x = drop 4 (show x) ++
+                                       if x `Set.member` pandocExtensions
+                                          then " +"
+                                          else " -"
+                     mapM_ (UTF8.hPutStrLn stdout . showExt)
+                               ([minBound..maxBound] :: [Extension])
+                     exitSuccess ))
+                 ""
+
+    , Option "" ["list-highlight-languages"]
+                 (NoArg
+                  (\_ -> do
+                     let langs = [ T.unpack (T.toLower (sShortname s))
+                                 | s <- M.elems defaultSyntaxMap
+                                 , sShortname s `notElem`
+                                    [T.pack "Alert", T.pack "Alert_indent"]
+                                 ]
+                     mapM_ (UTF8.hPutStrLn stdout) langs
+                     exitSuccess ))
+                 ""
+
+    , Option "" ["list-highlight-styles"]
+                 (NoArg
+                  (\_ -> do
+                     mapM_ (UTF8.hPutStrLn stdout) $
+                           map fst highlightingStyles
+                     exitSuccess ))
+                 ""
+
     , Option "v" ["version"]
                  (NoArg
                   (\_ -> do
                      prg <- getProgName
-                     defaultDatadir <- getAppUserDataDirectory "pandoc"
+                     defaultDatadir <- E.catch
+                            (getAppUserDataDirectory "pandoc")
+                            (\e -> let _ = (e :: E.SomeException)
+                                   in  return "")
                      UTF8.hPutStrLn stdout (prg ++ " " ++ pandocVersion ++
                        compileInfo ++ "\nDefault user data directory: " ++
                        defaultDatadir ++ copyrightMessage)
@@ -935,17 +1009,7 @@ readMetaValue s = case decode (UTF8.fromString s) of
 
 -- Returns usage message
 usageMessage :: String -> [OptDescr (Opt -> IO Opt)] -> String
-usageMessage programName = usageInfo
-  (programName ++ " [OPTIONS] [FILES]" ++ "\nInput formats:  " ++
-  wrapWords 16 78 readers'names ++
-     '\n' : replicate 16 ' ' ++
-     "[ *only Pandoc's JSON version of native AST]" ++ "\nOutput formats: " ++
-  wrapWords 16 78 writers'names ++
-     '\n' : replicate 16 ' ' ++
-     "[**for pdf output, use latex or beamer and -o FILENAME.pdf]\nOptions:")
-  where
-    writers'names = sort $ "json*" : "pdf**" : delete "json" (map fst writers)
-    readers'names = sort $ "json*" : delete "json" (map fst readers)
+usageMessage programName = usageInfo (programName ++ " [OPTIONS] [FILES]")
 
 -- Determine default reader based on source file extensions
 defaultReaderName :: String -> [FilePath] -> String
@@ -955,6 +1019,8 @@ defaultReaderName fallback (x:xs) =
     ".xhtml"    -> "html"
     ".html"     -> "html"
     ".htm"      -> "html"
+    ".md"       -> "markdown"
+    ".markdown" -> "markdown"
     ".tex"      -> "latex"
     ".latex"    -> "latex"
     ".ltx"      -> "latex"
@@ -1042,9 +1108,27 @@ adjustMetadata metadata d = return $ M.foldWithKey setMeta d metadata
 applyTransforms :: [Transform] -> Pandoc -> IO Pandoc
 applyTransforms transforms d = return $ foldr ($) d transforms
 
-applyFilters :: [FilePath] -> [String] -> Pandoc -> IO Pandoc
-applyFilters filters args d =
-  foldrM ($) d $ map (flip externalFilter args) filters
+  -- First we check to see if a filter is found.  If not, and if it's
+  -- not an absolute path, we check to see whether it's in `userdir/filters`.
+  -- If not, we leave it unchanged.
+expandFilterPath :: Maybe FilePath -> FilePath -> IO FilePath
+expandFilterPath mbDatadir fp = do
+  fpExists <- doesFileExist fp
+  if fpExists
+     then return fp
+     else case mbDatadir of
+               Just datadir | isRelative fp -> do
+                 let filterPath = (datadir </> "filters" </> fp)
+                 filterPathExists <- doesFileExist filterPath
+                 if filterPathExists
+                    then return filterPath
+                    else return fp
+               _ -> return fp
+
+applyFilters :: Maybe FilePath -> [FilePath] -> [String] -> Pandoc -> IO Pandoc
+applyFilters mbDatadir filters args d = do
+  expandedFilters <- mapM (expandFilterPath mbDatadir) filters
+  foldrM ($) d $ map (flip externalFilter args) expandedFilters
 
 uppercaseFirstLetter :: String -> String
 uppercaseFirstLetter (c:cs) = toUpper c : cs
@@ -1091,7 +1175,7 @@ convertWithOpts opts args = do
               , optHtmlQTags             = htmlQTags
               , optHighlight             = highlight
               , optHighlightStyle        = highlightStyle
-              , optChapters              = chapters
+              , optTopLevelDivision      = topLevelDivision
               , optHTMLMathMethod        = mathMethod'
               , optReferenceODT          = referenceODT
               , optReferenceDocx         = referenceDocx
@@ -1104,6 +1188,7 @@ convertWithOpts opts args = do
               , optIgnoreArgs            = ignoreArgs
               , optVerbose               = verbose
               , optReferenceLinks        = referenceLinks
+              , optReferenceLocation     = referenceLocation
               , optDpi                   = dpi
               , optWrapText              = wrap
               , optColumns               = columns
@@ -1134,7 +1219,7 @@ convertWithOpts opts args = do
        mapM_ (UTF8.hPutStrLn stdout) args
        exitSuccess
 
-  let csscdn = "https://cdnjs.cloudflare.com/ajax/libs/KaTeX/0.5.1/katex.min.css"
+  let csscdn = "https://cdnjs.cloudflare.com/ajax/libs/KaTeX/0.6.0/katex.min.css"
   let mathMethod =
         case (katexJS, katexStylesheet) of
             (Nothing, _) -> mathMethod'
@@ -1214,18 +1299,18 @@ convertWithOpts opts args = do
   let standalone' = standalone || not (isTextFormat format) || pdfOutput
 
   templ <- case templatePath of
-                _ | not standalone' -> return ""
+                _ | not standalone' -> return Nothing
                 Nothing -> do
                            deftemp <- getDefaultTemplate datadir format
                            case deftemp of
                                  Left e   -> throwIO e
-                                 Right t  -> return t
+                                 Right t  -> return (Just t)
                 Just tp -> do
                            -- strip off extensions
                            let tp' = case takeExtension tp of
                                           ""   -> tp <.> format
                                           _    -> tp
-                           E.catch (UTF8.readFile tp')
+                           Just <$> E.catch (UTF8.readFile tp')
                              (\e -> if isDoesNotExistError e
                                        then E.catch
                                              (readDataFileUTF8 datadir
@@ -1238,9 +1323,6 @@ convertWithOpts opts args = do
                       LaTeXMathML Nothing -> do
                          s <- readDataFileUTF8 datadir "LaTeXMathML.js"
                          return $ ("mathml-script", s) : variables
-                      MathML Nothing -> do
-                         s <- readDataFileUTF8 datadir "MathMLinHTML.js"
-                         return $ ("mathml-script", s) : variables
                       _ -> return variables
 
   variables'' <- if format == "dzslides"
@@ -1280,7 +1362,12 @@ convertWithOpts opts args = do
                       , readerFileScope   = fileScope
                       }
 
-  when (not (isTextFormat format) && outputFile == "-") $
+#ifdef _WINDOWS
+  let istty = True
+#else
+  istty <- queryTerminal stdOutput
+#endif
+  when (istty && not (isTextFormat format) && outputFile == "-") $
     err 5 $ "Cannot write " ++ format ++ " output to stdout.\n" ++
             "Specify an output file using the -o option."
 
@@ -1290,6 +1377,8 @@ convertWithOpts opts args = do
       readSource src = case parseURI src of
                             Just u | uriScheme u `elem` ["http:","https:"] ->
                                        readURI src
+                                   | uriScheme u == "file:" ->
+                                       UTF8.readFile (uriPath u)
                             _       -> UTF8.readFile src
       readURI src = do
         res <- openURL src
@@ -1331,8 +1420,7 @@ convertWithOpts opts args = do
     _  -> do pairs <- mapM (\s -> sourceToDoc [s]) sources
              return (mconcat $ map fst pairs, mconcat $ map snd pairs)
 
-  let writerOptions = def { writerStandalone       = standalone',
-                            writerTemplate         = templ,
+  let writerOptions = def { writerTemplate         = templ,
                             writerVariables        = variables'',
                             writerTabStop          = tabStop,
                             writerTableOfContents  = toc,
@@ -1344,6 +1432,7 @@ convertWithOpts opts args = do
                             writerNumberOffset     = numberFrom,
                             writerSectionDivs      = sectionDivs,
                             writerReferenceLinks   = referenceLinks,
+                            writerReferenceLocation = referenceLocation,
                             writerDpi              = dpi,
                             writerWrapText         = wrap,
                             writerColumns          = columns,
@@ -1353,7 +1442,7 @@ convertWithOpts opts args = do
                             writerUserDataDir      = datadir,
                             writerHtml5            = html5,
                             writerHtmlQTags        = htmlQTags,
-                            writerChapters         = chapters,
+                            writerTopLevelDivision = topLevelDivision,
                             writerListings         = listings,
                             writerBeamer           = False,
                             writerSlideLevel       = slideLevel,
@@ -1377,10 +1466,11 @@ convertWithOpts opts args = do
   doc' <- (maybe return (extractMedia media) mbExtractMedia >=>
            adjustMetadata metadata >=>
            applyTransforms transforms >=>
-           applyFilters filters' [format]) doc
+           applyFilters datadir filters' [format]) doc
 
-  let writeBinary :: B.ByteString -> IO ()
-      writeBinary = B.writeFile (UTF8.encodePath outputFile)
+  let writeFnBinary :: FilePath -> B.ByteString -> IO ()
+      writeFnBinary "-" = B.putStr
+      writeFnBinary f   = B.writeFile (UTF8.encodePath f)
 
   let writerFn :: FilePath -> String -> IO ()
       writerFn "-" = UTF8.putStr
@@ -1388,7 +1478,7 @@ convertWithOpts opts args = do
 
   case writer of
     IOStringWriter f -> f writerOptions doc' >>= writerFn outputFile
-    IOByteStringWriter f -> f writerOptions doc' >>= writeBinary
+    IOByteStringWriter f -> f writerOptions doc' >>= writeFnBinary outputFile
     PureStringWriter f
       | pdfOutput -> do
               -- make sure writer is latex or beamer or context or html5
@@ -1408,7 +1498,7 @@ convertWithOpts opts args = do
 
               res <- makePDF pdfprog f writerOptions doc'
               case res of
-                   Right pdf -> writeBinary pdf
+                   Right pdf -> writeFnBinary outputFile pdf
                    Left err' -> do
                      B.hPutStr stderr err'
                      B.hPut stderr $ B.pack [10]
