@@ -37,12 +37,11 @@ inline links:
 
 > module Main where
 > import Text.Pandoc
-> import Text.Pandoc.Error (handleError)
 >
 > markdownToRST :: String -> String
-> markdownToRST = handleError .
+> markdownToRST =
 >   writeRST def {writerReferenceLinks = True} .
->   readMarkdown def
+>   handleError . readMarkdown def
 >
 > main = getContents >>= putStrLn . markdownToRST
 
@@ -60,6 +59,8 @@ module Text.Pandoc
                , module Text.Pandoc.Generic
                -- * Options
                , module Text.Pandoc.Options
+               -- * Error handling
+               , module Text.Pandoc.Error
                -- * Lists of readers and writers
                , readers
                , writers
@@ -122,6 +123,7 @@ module Text.Pandoc
                -- * Miscellaneous
                , getReader
                , getWriter
+               , getDefaultExtensions
                , ToJsonFilter(..)
                , pandocVersion
              ) where
