@@ -33,7 +33,6 @@ module Text.Pandoc.Error (PandocError(..), handleError) where
 
 import Text.Parsec.Error
 import Text.Parsec.Pos hiding (Line)
-import Text.Pandoc.Compat.Except
 import GHC.Generics (Generic)
 import Data.Generics (Typeable)
 import Control.Exception (Exception)
@@ -48,10 +47,6 @@ data PandocError = -- | Generic parse failure
 
 instance Exception PandocError
 
-instance Error PandocError where
-  strMsg = ParseFailure
-
-
 -- | An unsafe method to handle `PandocError`s.
 handleError :: Either PandocError a -> a
 handleError (Right r) = r
@@ -62,8 +57,12 @@ handleError (Left err) =
         let errPos = errorPos err'
             errLine = sourceLine errPos
             errColumn = sourceColumn errPos
-            theline = (lines input ++ [""]) !! (errLine - 1)
-        in  error $ "\nError at " ++ show  err' ++ "\n" ++
-                theline ++ "\n" ++ replicate (errColumn - 1) ' ' ++
-                "^"
+            ls = lines input ++ [""]
+            errorInFile = if length ls > errLine - 1
+                            then concat ["\n", (ls !! (errLine - 1))
+                                        ,"\n", replicate (errColumn - 1) ' '
+                                        ,"^"]
+                        else ""
+        in  error $ "\nError at " ++ show  err'
+                ++ errorInFile
 
