@@ -498,8 +498,8 @@ mimeTypesList = -- List borrowed from happstack-server.
            ,("wmv","video/x-ms-wmv")
            ,("wmx","video/x-ms-wmx")
            ,("wmz","application/x-ms-wmz")
-           ,("woff","application/x-font-woff")
-           ,("woff2","application/x-font-woff2")
+           ,("woff","application/font-woff")
+           ,("woff2","font/woff2")
            ,("wp5","application/wordperfect5.1")
            ,("wpd","application/wordperfect")
            ,("wrl","model/vrml")
