@@ -5,7 +5,7 @@ import Text.Pandoc.Options
 import Text.Pandoc.Definition
 import Text.Pandoc.Builder
 import Text.XML.Light
-import Text.Pandoc.Compat.TagSoupEntity (lookupEntity)
+import Text.HTML.TagSoup.Entity (lookupEntity)
 import Data.Either (rights)
 import Data.Generics
 import Data.Char (isSpace)
@@ -14,7 +14,7 @@ import Data.List (intersperse)
 import Data.Maybe (fromMaybe)
 import Text.TeXMath (readMathML, writeTeX)
 import Text.Pandoc.Error (PandocError)
-import Text.Pandoc.Compat.Except
+import Control.Monad.Except
 import Data.Default
 import Data.Foldable (asum)
 
@@ -564,7 +564,7 @@ normalizeTree = everywhere (mkT go)
         go xs = xs
 
 convertEntity :: String -> String
-convertEntity e = maybe (map toUpper e) (:[]) (lookupEntity e)
+convertEntity e = maybe (map toUpper e) id (lookupEntity e)
 
 -- convenience function to get an attribute value, defaulting to ""
 attrValue :: String -> Element -> String
@@ -592,8 +592,6 @@ checkInMeta p = do
   when accepts p
   return mempty
 
-
-
 addMeta :: ToMetaValue a => String -> a -> DB ()
 addMeta field val = modify (setMeta field val)
 
@@ -612,7 +610,7 @@ isBlockElement (Elem e) = qName (elName e) `elem` blocktags
            "important","caution","note","tip","warning","qandadiv",
            "question","answer","abstract","itemizedlist","orderedlist",
            "variablelist","article","book","table","informaltable",
-           "informalexample",
+           "informalexample", "linegroup",
            "screen","programlisting","example","calloutlist"]
 isBlockElement _ = False
 
@@ -779,6 +777,7 @@ parseBlock (Elem e) =
         "informaltable" -> parseTable
         "informalexample" -> divWith ("", ["informalexample"], []) <$>
                              getBlocks e
+        "linegroup" -> lineBlock <$> lineItems
         "literallayout" -> codeBlockWithLang
         "screen" -> codeBlockWithLang
         "programlisting" -> codeBlockWithLang
@@ -900,6 +899,7 @@ parseBlock (Elem e) =
                      let ident = attrValue "id" e
                      modify $ \st -> st{ dbSectionLevel = n - 1 }
                      return $ headerWith (ident,[],[]) n' headerText <> b
+         lineItems = mapM getInlines $ filterChildren (named "line") e
          metaBlock = acceptingMetadata (getBlocks e) >> return mempty
 
 getInlines :: Element -> DB Inlines
@@ -916,7 +916,7 @@ elementToStr x = x
 parseInline :: Content -> DB Inlines
 parseInline (Text (CData _ s _)) = return $ text s
 parseInline (CRef ref) =
-  return $ maybe (text $ map toUpper ref) (text . (:[])) $ lookupEntity ref
+  return $ maybe (text $ map toUpper ref) (text) $ lookupEntity ref
 parseInline (Elem e) =
   case qName (elName e) of
         "equation" -> equation displayMath
@@ -964,9 +964,11 @@ parseInline (Elem e) =
             content <- dbContent <$> get
             let linkend = attrValue "linkend" e
             let title = case attrValue "endterm" e of
-                            ""      -> maybe "???" xrefTitleByElem (findElementById linkend content)
-                            endterm -> maybe "???" strContent (findElementById endterm content)
-            return $ link ('#' : linkend) "" (singleton (Str title))
+                            ""      -> maybe "???" xrefTitleByElem
+                                         (findElementById linkend content)
+                            endterm -> maybe "???" strContent
+                                         (findElementById endterm content)
+            return $ link ('#' : linkend) "" (text title)
         "email" -> return $ link ("mailto:" ++ strContent e) ""
                           $ str $ strContent e
         "uri" -> return $ link (strContent e) "" $ str $ strContent e
@@ -1050,4 +1052,4 @@ parseInline (Elem e) =
           where
             xrefLabel = attrValue "xreflabel" el
             descendantContent name = maybe "???" strContent
-                                   . findElement (QName name Nothing Nothing)
+                                   . filterElementName (\n -> qName n == name)
