@@ -10,16 +10,17 @@ module Text.Pandoc.Readers.EPUB
 
 import Text.XML.Light
 import Text.Pandoc.Definition hiding (Attr)
-import Text.Pandoc.Walk (walk, query)
 import Text.Pandoc.Readers.HTML (readHtml)
+import Text.Pandoc.Error
+import Text.Pandoc.Walk (walk, query)
 import Text.Pandoc.Options ( ReaderOptions(..), readerTrace)
 import Text.Pandoc.Shared (escapeURI, collapseFilePath, addMetaField)
+import Network.URI (unEscapeString)
 import Text.Pandoc.MediaBag (MediaBag, insertMedia)
-import Text.Pandoc.Compat.Except (MonadError, throwError, runExcept, Except)
-import Text.Pandoc.Compat.Monoid ((<>))
+import Control.Monad.Except (MonadError, throwError, runExcept, Except)
 import Text.Pandoc.MIME (MimeType)
 import qualified Text.Pandoc.Builder as B
-import Codec.Archive.Zip ( Archive (..), toArchive, fromEntry
+import Codec.Archive.Zip ( Archive (..), toArchiveOrFail, fromEntry
                          , findEntryByPath, Entry)
 import qualified Data.ByteString.Lazy as BL (ByteString)
 import System.FilePath ( takeFileName, (</>), dropFileName, normalise
@@ -30,16 +31,17 @@ import Control.Monad (guard, liftM, when)
 import Data.List (isPrefixOf, isInfixOf)
 import Data.Maybe (mapMaybe, fromMaybe)
 import qualified Data.Map as M (Map, lookup, fromList, elems)
+import Data.Monoid ((<>))
 import Control.DeepSeq (deepseq, NFData)
 
 import Debug.Trace (trace)
 
-import Text.Pandoc.Error
-
 type Items = M.Map String (FilePath, MimeType)
 
 readEPUB :: ReaderOptions -> BL.ByteString -> Either PandocError (Pandoc, MediaBag)
-readEPUB opts bytes = runEPUB (archiveToEPUB opts $ toArchive bytes)
+readEPUB opts bytes = case toArchiveOrFail bytes of
+  Right archive -> runEPUB $ archiveToEPUB opts $ archive
+  Left  _       -> Left $ ParseFailure "Couldn't extract ePub file"
 
 runEPUB :: Except PandocError a -> Either PandocError a
 runEPUB = runExcept
@@ -72,14 +74,15 @@ archiveToEPUB os archive = do
       let docSpan = B.doc $ B.para $ B.spanWith (takeFileName path, [], []) mempty
       return $ docSpan <> doc
     mimeToReader :: MonadError PandocError m => MimeType -> FilePath -> FilePath -> m Pandoc
-    mimeToReader "application/xhtml+xml" (normalise -> root) (normalise -> path) = do
+    mimeToReader "application/xhtml+xml" (unEscapeString -> root)
+                                         (unEscapeString -> path) = do
       fname <- findEntryByPathE (root </> path) archive
       html <- either throwError return .
                 readHtml os' .
                   UTF8.toStringLazy $
                     fromEntry fname
       return $ fixInternalReferences path html
-    mimeToReader s _ path
+    mimeToReader s _ (unEscapeString -> path)
       | s `elem` imageMimes = return $ imageToPandoc path
       | otherwise = return $ mempty
 
@@ -105,7 +108,9 @@ iq _ = []
 
 -- Remove relative paths
 renameImages :: FilePath -> Inline -> Inline
-renameImages root (Image attr a (url, b)) = Image attr a (collapseFilePath (root </> url), b)
+renameImages root img@(Image attr a (url, b))
+  | "data:" `isPrefixOf` url = img
+  | otherwise                = Image attr a (collapseFilePath (root </> url), b)
 renameImages _ x = x
 
 imageToPandoc :: FilePath -> Pandoc
@@ -190,8 +195,10 @@ fixInlineIRs s (Span as v) =
   Span (fixAttrs s as) v
 fixInlineIRs s (Code as code) =
   Code (fixAttrs s as) code
-fixInlineIRs s (Link attr t ('#':url, tit)) =
-  Link attr t (addHash s url, tit)
+fixInlineIRs s (Link as is ('#':url, tit)) =
+  Link (fixAttrs s as) is (addHash s url, tit)
+fixInlineIRs s (Link as is t) =
+  Link (fixAttrs s as) is t
 fixInlineIRs _ v = v
 
 prependHash :: [String] -> Inline -> Inline
