@@ -232,7 +232,7 @@ inline = (mempty <$ comment)
      <|> (guardEnabled Ext_literate_haskell *> char '|' *> doLHSverb)
      <|> (str . (:[]) <$> tildeEscape)
      <|> (str . (:[]) <$> oneOf "[]")
-     <|> (str . (:[]) <$> oneOf "#&") -- TODO print warning?
+     <|> (str . (:[]) <$> oneOf "#&~^'`\"[]") -- TODO print warning?
      -- <|> (str <$> count 1 (satisfy (\c -> c /= '\\' && c /='\n' && c /='}' && c /='{'))) -- eat random leftover characters
 
 inlines :: LP Inlines
@@ -340,6 +340,7 @@ blockCommands = M.fromList $
   , ("closing", skipopts *> closing)
   --
   , ("hrule", pure horizontalRule)
+  , ("strut", pure mempty)
   , ("rule", skipopts *> tok *> tok *> pure horizontalRule)
   , ("item", skipopts *> looseItem)
   , ("documentclass", skipopts *> braced *> preamble)
@@ -366,7 +367,6 @@ blockCommands = M.fromList $
   , "ignore"
   , "hyperdef"
   , "markboth", "markright", "markleft"
-  , "hspace", "vspace"
   , "newpage"
   ]
 
@@ -422,7 +422,8 @@ inlineCommand = try $ do
   star <- option "" (string "*")
   let name' = name ++ star
   let raw = do
-        rawargs <- withRaw (skipopts *> option "" dimenarg *> many braced)
+        rawargs <- withRaw
+               (skipangles *> skipopts *> option "" dimenarg *> many braced)
         let rawcommand = '\\' : name ++ star ++ snd rawargs
         transformed <- applyMacros' rawcommand
         if transformed /= rawcommand
@@ -523,6 +524,7 @@ inlineCommands = M.fromList $
   , ("copyright", lit "©")
   , ("textasciicircum", lit "^")
   , ("textasciitilde", lit "~")
+  , ("H", try $ tok >>= accent hungarumlaut)
   , ("`", option (str "`") $ try $ tok >>= accent grave)
   , ("'", option (str "'") $ try $ tok >>= accent acute)
   , ("^", option (str "^") $ try $ tok >>= accent circ)
@@ -762,6 +764,21 @@ umlaut 'o' = "ö"
 umlaut 'u' = "ü"
 umlaut c   = [c]
 
+hungarumlaut :: Char -> String
+hungarumlaut 'A' = "A̋"
+hungarumlaut 'E' = "E̋"
+hungarumlaut 'I' = "I̋"
+hungarumlaut 'O' = "Ő"
+hungarumlaut 'U' = "Ű"
+hungarumlaut 'Y' = "ӳ"
+hungarumlaut 'a' = "a̋"
+hungarumlaut 'e' = "e̋"
+hungarumlaut 'i' = "i̋"
+hungarumlaut 'o' = "ő"
+hungarumlaut 'u' = "ű"
+hungarumlaut 'y' = "ӳ"
+hungarumlaut c   = [c]
+
 dot :: Char -> String
 dot 'C' = "Ċ"
 dot 'c' = "ċ"
@@ -859,8 +876,26 @@ tok = try $ grouped inline <|> inlineCommand <|> str <$> count 1 inlineChar
 opt :: LP Inlines
 opt = bracketed inline
 
+rawopt :: LP String
+rawopt = do
+  contents <- bracketed (many1 (noneOf "[]") <|> try (string "\\]") <|>
+                   try (string "\\[") <|> rawopt)
+  optional sp
+  return $ "[" ++ contents ++ "]"
+
 skipopts :: LP ()
-skipopts = skipMany (opt *> optional sp)
+skipopts = skipMany rawopt
+
+-- opts in angle brackets are used in beamer
+rawangle :: LP ()
+rawangle = try $ do
+  char '<'
+  skipMany (noneOf ">")
+  char '>'
+  return ()
+
+skipangles :: LP ()
+skipangles = skipMany rawangle
 
 inlineText :: LP Inlines
 inlineText = str <$> many1 inlineChar
@@ -883,8 +918,9 @@ inlineEnvironment = try $ do
 
 rawEnv :: String -> LP Blocks
 rawEnv name = do
-  let addBegin x = "\\begin{" ++ name ++ "}" ++ x
   parseRaw <- getOption readerParseRaw
+  rawOptions <- mconcat <$> many rawopt
+  let addBegin x = "\\begin{" ++ name ++ "}" ++ rawOptions ++ x
   if parseRaw
      then (rawBlock "latex" . addBegin) <$>
             (withRaw (env name blocks) >>= applyMacros' . snd)
@@ -925,8 +961,8 @@ verbatimEnv' = fmap snd <$>
   withRaw $ try $ do
              string "\\begin"
              name <- braced'
-             guard $ name `elem` ["verbatim", "Verbatim", "lstlisting",
-                                  "minted", "alltt", "comment"]
+             guard $ name `elem` ["verbatim", "Verbatim", "BVerbatim",
+                                  "lstlisting", "minted", "alltt", "comment"]
              manyTill anyChar (try $ string $ "\\end{" ++ name ++ "}")
 
 blob' :: IncludeParser
@@ -1060,11 +1096,13 @@ environments = M.fromList
   [ ("document", env "document" blocks <* skipMany anyChar)
   , ("abstract", mempty <$ (env "abstract" blocks >>= addMeta "abstract"))
   , ("letter", env "letter" letterContents)
+  , ("minipage", env "minipage" $
+         skipopts *> spaces' *> optional braced *> spaces' *> blocks)
   , ("figure", env "figure" $
          resetCaption *> skipopts *> blocks >>= addImageCaption)
   , ("center", env "center" blocks)
   , ("longtable",  env "longtable" $
-         resetCaption *> skipopts *> blocks >>= addTableCaption)
+         resetCaption *> simpTable False >>= addTableCaption)
   , ("table",  env "table" $
          resetCaption *> skipopts *> blocks >>= addTableCaption)
   , ("tabular*", env "tabular" $ simpTable True)
@@ -1081,14 +1119,8 @@ environments = M.fromList
         verbEnv "code"))
   , ("comment", mempty <$ verbEnv "comment")
   , ("verbatim", codeBlock <$> verbEnv "verbatim")
-  , ("Verbatim",   do options <- option [] keyvals
-                      let kvs = [ (if k == "firstnumber"
-                                      then "startFrom"
-                                      else k, v) | (k,v) <- options ]
-                      let classes = [ "numberLines" |
-                                      lookup "numbers" options == Just "left" ]
-                      let attr = ("",classes,kvs)
-                      codeBlockWith attr <$> verbEnv "Verbatim")
+  , ("Verbatim", fancyverbEnv "Verbatim")
+  , ("BVerbatim", fancyverbEnv "BVerbatim")
   , ("lstlisting", do options <- option [] keyvals
                       let kvs = [ (if k == "firstnumber"
                                       then "startFrom"
@@ -1199,6 +1231,17 @@ verbEnv name = do
   res <- manyTill anyChar endEnv
   return $ stripTrailingNewlines res
 
+fancyverbEnv :: String -> LP Blocks
+fancyverbEnv name = do
+  options <- option [] keyvals
+  let kvs = [ (if k == "firstnumber"
+                  then "startFrom"
+                  else k, v) | (k,v) <- options ]
+  let classes = [ "numberLines" |
+                  lookup "numbers" options == Just "left" ]
+  let attr = ("",classes,kvs)
+  codeBlockWith attr <$> verbEnv name
+
 orderedList' :: LP Blocks
 orderedList' = do
   optional sp
@@ -1275,7 +1318,7 @@ citationLabel  = optional sp *>
           <* optional sp
           <* optional (char ',')
           <* optional sp)
-  where isBibtexKeyChar c = isAlphaNum c || c `elem` (".:;?!`'()/*@_+=-[]*" :: String)
+  where isBibtexKeyChar c = isAlphaNum c || c `elem` (".:;?!`'()/*@_+=-[]" :: String)
 
 cites :: CitationMode -> Bool -> LP [Citation]
 cites mode multi = try $ do
@@ -1339,22 +1382,30 @@ hline = try $ do
     -- booktabs rules:
     controlSeq "toprule" <|>
     controlSeq "bottomrule" <|>
-    controlSeq "midrule"
+    controlSeq "midrule" <|>
+    controlSeq "endhead" <|>
+    controlSeq "endfirsthead"
   spaces'
   optional $ bracketed (many1 (satisfy (/=']')))
   return ()
 
 lbreak :: LP ()
-lbreak = () <$ try (spaces' *> controlSeq "\\" <* spaces')
+lbreak = () <$ try (spaces' *>
+                    (controlSeq "\\" <|> controlSeq "tabularnewline") <*
+                    spaces')
 
 amp :: LP ()
-amp = () <$ try (spaces' *> char '&')
+amp = () <$ try (spaces' *> char '&' <* spaces')
 
 parseTableRow :: Int  -- ^ number of columns
               -> LP [Blocks]
 parseTableRow cols = try $ do
   let tableCellInline = notFollowedBy (amp <|> lbreak) >> inline
-  let tableCell = (plain . trimInlines . mconcat) <$> many tableCellInline
+  let minipage = try $ controlSeq "begin" *> string "{minipage}" *>
+          env "minipage"
+          (skipopts *> spaces' *> optional braced *> spaces' *> blocks)
+  let tableCell = minipage <|>
+            ((plain . trimInlines . mconcat) <$> many tableCellInline)
   cells' <- sepBy1 tableCell amp
   let numcells = length cells'
   guard $ numcells <= cols && numcells >= 1
@@ -1373,9 +1424,17 @@ simpTable hasWidthParameter = try $ do
   skipopts
   aligns <- parseAligns
   let cols = length aligns
-  optional hline
-  header' <- option [] $ try (parseTableRow cols <* lbreak <* hline)
-  rows <- sepEndBy (parseTableRow cols) (lbreak <* optional hline)
+  optional $ controlSeq "caption" *> skipopts *> setCaption
+  optional lbreak
+  spaces'
+  skipMany hline
+  spaces'
+  header' <- option [] $ try (parseTableRow cols <* lbreak <* many1 hline)
+  spaces'
+  rows <- sepEndBy (parseTableRow cols) (lbreak <* optional (skipMany hline))
+  spaces'
+  optional $ controlSeq "caption" *> skipopts *> setCaption
+  optional lbreak
   spaces'
   let header'' = if null header'
                     then replicate cols mempty
