@@ -39,7 +39,7 @@ module Text.Pandoc.Readers.MediaWiki ( readMediaWiki ) where
 import Text.Pandoc.Definition
 import qualified Text.Pandoc.Builder as B
 import Text.Pandoc.Builder (Inlines, Blocks, trimInlines)
-import Text.Pandoc.Compat.Monoid ((<>))
+import Data.Monoid ((<>))
 import Text.Pandoc.Options
 import Text.Pandoc.Readers.HTML ( htmlTag, isBlockTag, isCommentTag )
 import Text.Pandoc.XML ( fromEntities )
@@ -253,7 +253,7 @@ parseAttr = try $ do
   k <- many1 letter
   char '='
   v <- (char '"' >> many1Till (satisfy (/='\n')) (char '"'))
-       <|> many1 nonspaceChar
+       <|> many1 (satisfy $ \c -> not (isSpace c) && c /= '|')
   return (k,v)
 
 tableStart :: MWParser ()
@@ -376,11 +376,17 @@ preformatted = try $ do
       spacesStr _        = False
   if F.all spacesStr contents
      then return mempty
-     else return $ B.para $ walk strToCode contents
+     else return $ B.para $ encode contents
 
-strToCode :: Inline -> Inline
-strToCode (Str s) = Code ("",[],[]) s
-strToCode  x      = x
+encode :: Inlines -> Inlines
+encode = B.fromList . normalizeCode . B.toList . walk strToCode
+  where strToCode (Str s) = Code ("",[],[]) s
+        strToCode Space   = Code ("",[],[]) " "
+        strToCode  x      = x
+        normalizeCode []  = []
+        normalizeCode (Code a1 x : Code a2 y : zs) | a1 == a2 =
+          normalizeCode $ (Code a1 (x ++ y)) : zs
+        normalizeCode (x:xs) = x : normalizeCode xs
 
 header :: MWParser Blocks
 header = try $ do
@@ -417,7 +423,8 @@ defListItem = try $ do
   terms <- mconcat . intersperse B.linebreak <$> many defListTerm
   -- we allow dd with no dt, or dt with no dd
   defs  <- if B.isNull terms
-              then notFollowedBy (try $ string ":<math>") *>
+              then notFollowedBy
+                    (try $ skipMany1 (char ':') >> string "<math>") *>
                        many1 (listItem ':')
               else many (listItem ':')
   return (terms, defs)
@@ -513,7 +520,7 @@ str :: MWParser Inlines
 str = B.str <$> many1 (noneOf $ specialChars ++ spaceChars)
 
 math :: MWParser Inlines
-math = (B.displayMath . trim <$> try (char ':' >> charsInTags "math"))
+math = (B.displayMath . trim <$> try (many1 (char ':') >> charsInTags "math"))
    <|> (B.math . trim <$> charsInTags "math")
    <|> (B.displayMath . trim <$> try (dmStart *> manyTill anyChar dmEnd))
    <|> (B.math . trim <$> try (mStart *> manyTill (satisfy (/='\n')) mEnd))
@@ -545,8 +552,8 @@ inlineTag = do
        TagOpen "del" _ -> B.strikeout <$> inlinesInTags "del"
        TagOpen "sub" _ -> B.subscript <$> inlinesInTags "sub"
        TagOpen "sup" _ -> B.superscript <$> inlinesInTags "sup"
-       TagOpen "code" _ -> walk strToCode <$> inlinesInTags "code"
-       TagOpen "tt" _ -> walk strToCode <$> inlinesInTags "tt"
+       TagOpen "code" _ -> encode <$> inlinesInTags "code"
+       TagOpen "tt" _ -> encode <$> inlinesInTags "tt"
        TagOpen "hask" _ -> B.codeWith ("",["haskell"],[]) <$> charsInTags "hask"
        _ -> B.rawInline "html" . snd <$> htmlTag (~== tag)
 
@@ -579,7 +586,7 @@ image :: MWParser Inlines
 image = try $ do
   sym "[["
   choice imageIdentifiers
-  fname <- many1 (noneOf "|]")
+  fname <- addUnderscores <$> many1 (noneOf "|]")
   _ <- many imageOption
   dims <- try (char '|' *> (sepBy (many digit) (char 'x')) <* string "px")
           <|> return []
@@ -665,9 +672,6 @@ strong = B.strong <$> nested (inlinesBetween start end)
           end   = try $ sym "'''"
 
 doubleQuotes :: MWParser Inlines
-doubleQuotes = B.doubleQuoted . trimInlines . mconcat <$> try
- ((getState >>= guard . readerSmart . mwOptions) *>
-   openDoubleQuote *> manyTill inline closeDoubleQuote )
-    where openDoubleQuote = char '"' <* lookAhead alphaNum
-          closeDoubleQuote = char '"' <* notFollowedBy alphaNum
-
+doubleQuotes = B.doubleQuoted <$> nested (inlinesBetween openDoubleQuote closeDoubleQuote)
+    where openDoubleQuote = sym "\"" >> lookAhead nonspaceChar
+          closeDoubleQuote = try $ sym "\""
