@@ -33,17 +33,16 @@ module Text.Pandoc.Readers.Org.Inlines
   , linkTarget
   ) where
 
-import           Text.Pandoc.Readers.Org.BlockStarts
+import           Text.Pandoc.Readers.Org.BlockStarts ( endOfBlock, noteMarker )
 import           Text.Pandoc.Readers.Org.ParserState
 import           Text.Pandoc.Readers.Org.Parsing
 import           Text.Pandoc.Readers.Org.Shared
-                   ( isImageFilename, rundocBlockClass, toRundocAttrib
-                   , translateLang )
+                   ( cleanLinkString, isImageFilename, rundocBlockClass
+                   , toRundocAttrib, translateLang )
 
 import qualified Text.Pandoc.Builder as B
 import           Text.Pandoc.Builder ( Inlines )
 import           Text.Pandoc.Definition
-import           Text.Pandoc.Compat.Monoid ( (<>) )
 import           Text.Pandoc.Options
 import           Text.Pandoc.Readers.LaTeX ( inlineCommand, rawLaTeXInline )
 import           Text.TeXMath ( readTeX, writePandoc, DisplayType(..) )
@@ -52,9 +51,10 @@ import qualified Text.TeXMath.Readers.MathML.EntityMap as MathMLEntityMap
 import           Prelude hiding (sequence)
 import           Control.Monad ( guard, mplus, mzero, when, void )
 import           Data.Char ( isAlphaNum, isSpace )
-import           Data.List ( intersperse, isPrefixOf )
+import           Data.List ( intersperse )
 import           Data.Maybe ( fromMaybe )
 import qualified Data.Map as M
+import           Data.Monoid ( (<>) )
 import           Data.Traversable (sequence)
 
 --
@@ -152,18 +152,7 @@ str = return . B.str <$> many1 (noneOf $ specialChars ++ "\n\r ")
 endline :: OrgParser (F Inlines)
 endline = try $ do
   newline
-  notFollowedBy blankline
-  notFollowedBy' exampleLineStart
-  notFollowedBy' hline
-  notFollowedBy' noteMarker
-  notFollowedBy' tableStart
-  notFollowedBy' drawerStart
-  notFollowedBy' headerStart
-  notFollowedBy' metaLineStart
-  notFollowedBy' latexEnvStart
-  notFollowedBy' commentLineStart
-  notFollowedBy' bulletListStart
-  notFollowedBy' orderedListStart
+  notFollowedBy' endOfBlock
   decEmphasisNewlinesCount
   guard =<< newlinesCountWithinLimits
   updateLastPreCharPos
@@ -209,21 +198,22 @@ orgRefCite = try $ choice
 normalOrgRefCite :: OrgParser (F [Citation])
 normalOrgRefCite = try $ do
   mode <- orgRefCiteMode
-  sequence <$> sepBy1 (orgRefCiteList mode) (char ',')
- where
-   -- | A list of org-ref style citation keys, parsed as citation of the given
-   -- citation mode.
-   orgRefCiteList :: CitationMode -> OrgParser (F Citation)
-   orgRefCiteList citeMode = try $ do
-     key <- orgRefCiteKey
-     returnF $ Citation
-      { citationId      = key
-      , citationPrefix  = mempty
-      , citationSuffix  = mempty
-      , citationMode    = citeMode
-      , citationNoteNum = 0
-      , citationHash    = 0
-      }
+  -- org-ref style citation key, parsed into a citation of the given mode
+  let orgRefCiteItem :: OrgParser (F Citation)
+      orgRefCiteItem = try $ do
+        key <- orgRefCiteKey
+        returnF $ Citation
+          { citationId      = key
+          , citationPrefix  = mempty
+          , citationSuffix  = mempty
+          , citationMode    = mode
+          , citationNoteNum = 0
+          , citationHash    = 0
+          }
+  firstCitation <- orgRefCiteItem
+  moreCitations <- many (try $ char ',' *> orgRefCiteItem)
+  return . sequence $ firstCitation : moreCitations
+    where
 
 -- | Read an Berkeley-style Org-mode citation.  Berkeley citation style was
 -- develop and adjusted to Org-mode style by John MacFarlane and Richard
@@ -435,9 +425,11 @@ explicitOrImageLink = try $ do
   char ']'
   return $ do
     src <- srcF
-    if isImageFilename title
-      then pure $ B.link src "" $ B.image title mempty mempty
-      else linkToInlinesF src =<< title'
+    case cleanLinkString title of
+      Just imgSrc | isImageFilename imgSrc ->
+        pure $ B.link src "" $ B.image imgSrc mempty mempty
+      _ ->
+        linkToInlinesF src =<< title'
 
 selflinkOrImage :: OrgParser (F Inlines)
 selflinkOrImage = try $ do
@@ -482,25 +474,6 @@ linkToInlinesF linkStr =
                                        else pure . B.link cleanedLink ""
                  Nothing -> internalLink linkStr  -- other internal link
 
--- | Cleanup and canonicalize a string describing a link.  Return @Nothing@ if
--- the string does not appear to be a link.
-cleanLinkString :: String -> Maybe String
-cleanLinkString s =
-  case s of
-    '/':_                  -> Just $ "file://" ++ s  -- absolute path
-    '.':'/':_              -> Just s                 -- relative path
-    '.':'.':'/':_          -> Just s                 -- relative path
-    -- Relative path or URL (file schema)
-    'f':'i':'l':'e':':':s' -> Just $ if ("//" `isPrefixOf` s') then s else s'
-    _ | isUrl s            -> Just s                 -- URL
-    _                      -> Nothing
- where
-   isUrl :: String -> Bool
-   isUrl cs =
-     let (scheme, path) = break (== ':') cs
-     in all (\c -> isAlphaNum c || c `elem` (".-"::String)) scheme
-          && not (null path)
-
 internalLink :: String -> Inlines -> F Inlines
 internalLink link title = do
   anchorB <- (link `elem`) <$> asksF orgStateAnchorIds
@@ -675,6 +648,9 @@ emphasisStart c = try $ do
   char c
   lookAhead (noneOf emphasisForbiddenBorderChars)
   pushToInlineCharStack c
+  -- nested inlines are allowed, so mark this position as one which might be
+  -- followed by another inline.
+  updateLastPreCharPos
   return c
 
 -- | Parses the closing character of emphasis
