@@ -39,7 +39,7 @@ import Text.Pandoc.Shared
 import Text.Pandoc.Parsing
 import Text.Pandoc.Options
 import Control.Monad ( when, liftM, guard, mzero )
-import Data.List ( findIndex, intersperse, intercalate,
+import Data.List ( findIndex, intercalate,
                    transpose, sort, deleteFirstsBy, isSuffixOf , nub, union)
 import Data.Maybe (fromMaybe)
 import qualified Data.Map as M
@@ -48,7 +48,7 @@ import Text.Pandoc.Builder (Inlines, Blocks, trimInlines)
 import qualified Text.Pandoc.Builder as B
 import Data.Sequence (viewr, ViewR(..))
 import Data.Char (toLower, isHexDigit, isSpace)
-import Text.Pandoc.Compat.Monoid ((<>))
+import Data.Monoid ((<>))
 import Text.Pandoc.Error
 
 -- | Parse reStructuredText string and return Pandoc document.
@@ -228,7 +228,7 @@ lineBlock :: RSTParser Blocks
 lineBlock = try $ do
   lines' <- lineBlockLines
   lines'' <- mapM parseInlineFromString lines'
-  return $ B.para (mconcat $ intersperse B.linebreak lines'')
+  return $ B.lineBlock lines''
 
 --
 -- paragraph block
@@ -562,13 +562,11 @@ directive' = do
         "rubric" -> B.para . B.strong <$> parseInlineFromString top
         _ | label `elem` ["attention","caution","danger","error","hint",
                           "important","note","tip","warning"] ->
-           do let tit = B.para $ B.strong $ B.str label
-              bod <- parseFromString parseBlocks $ top ++ "\n\n" ++ body'
-              return $ B.blockQuote $ tit <> bod
+           do bod <- parseFromString parseBlocks $ top ++ "\n\n" ++ body'
+              return $ B.divWith ("",["admonition", label],[]) bod
         "admonition" ->
-           do tit <- B.para . B.strong <$> parseInlineFromString top
-              bod <- parseFromString parseBlocks body'
-              return $ B.blockQuote $ tit <> bod
+           do bod <- parseFromString parseBlocks $ top ++ "\n\n" ++ body'
+              return $ B.divWith ("",["admonition"],[]) bod
         "sidebar" ->
            do let subtit = maybe "" trim $ lookup "subtitle" fields
               tit <- B.para . B.strong <$> parseInlineFromString
@@ -576,11 +574,11 @@ directive' = do
                                           then ""
                                           else (":  " ++ subtit))
               bod <- parseFromString parseBlocks body'
-              return $ B.blockQuote $ tit <> bod
+              return $ B.divWith ("",["sidebar"],[]) $ tit <> bod
         "topic" ->
            do tit <- B.para . B.strong <$> parseInlineFromString top
               bod <- parseFromString parseBlocks body'
-              return $ tit <> bod
+              return $ B.divWith ("",["topic"],[]) $ tit <> bod
         "default-role" -> mempty <$ updateState (\s ->
                               s { stateRstDefaultRole =
                                   case trim top of
@@ -951,7 +949,8 @@ table = gridTable False <|> simpleTable False <|>
 --
 
 inline :: RSTParser Inlines
-inline = choice [ whitespace
+inline = choice [ note          -- can start with whitespace, so try before ws
+                , whitespace
                 , link
                 , str
                 , endline
@@ -960,7 +959,6 @@ inline = choice [ whitespace
                 , code
                 , subst
                 , interpretedRole
-                , note
                 , smart
                 , hyphens
                 , escapedChar
@@ -1122,14 +1120,28 @@ explicitLink = try $ do
   notFollowedBy (char '`') -- `` marks start of inline code
   label' <- trimInlines . mconcat <$>
              manyTill (notFollowedBy (char '`') >> inline) (char '<')
-  src <- manyTill (noneOf ">\n") (char '>')
+  src <- trim <$> manyTill (noneOf ">\n") (char '>')
   skipSpaces
   string "`_"
   optional $ char '_' -- anonymous form
   let label'' = if label' == mempty
                    then B.str src
                    else label'
-  return $ B.link (escapeURI $ trim src) "" label''
+  -- `link <google_>` is a reference link to _google!
+  (src',tit,attr) <- case reverse src of
+                          '_':xs -> do
+                            keyTable <- stateKeys <$> getState
+                            let key = toKey $ reverse xs
+                            case M.lookup key keyTable of
+                                 Nothing -> do
+                                   pos <- getPosition
+                                   addWarning (Just pos) $
+                                     "Could not find reference for " ++
+                                       show key
+                                   return ("","",nullAttr)
+                                 Just ((s,t),a) -> return (s,t,a)
+                          _ -> return (src, "", nullAttr)
+  return $ B.linkWith attr (escapeURI src') tit label''
 
 referenceLink :: RSTParser Inlines
 referenceLink = try $ do
@@ -1146,7 +1158,12 @@ referenceLink = try $ do
                       then mzero
                       else return (head anonKeys)
   ((src,tit), attr) <- case M.lookup key keyTable of
-                         Nothing  -> fail "no corresponding key"
+                         Nothing  -> do
+                           pos <- getPosition
+                           addWarning (Just pos) $
+                             "Could not find reference for " ++
+                               show key
+                           return (("",""),nullAttr)
                          Just val -> return val
   -- if anonymous link, remove key so it won't be used again
   when (isAnonKey key) $ updateState $ \s -> s{ stateKeys = M.delete key keyTable }
@@ -1170,18 +1187,28 @@ subst = try $ do
   (_,ref) <- withRaw $ enclosed (char '|') (char '|') inline
   state <- getState
   let substTable = stateSubstitutions state
-  case M.lookup (toKey $ stripFirstAndLast ref) substTable of
-       Nothing     -> fail "no corresponding key"
+  let key = toKey $ stripFirstAndLast ref
+  case M.lookup key substTable of
+       Nothing     -> do
+         pos <- getPosition
+         addWarning (Just pos) $
+           "Could not find reference for " ++ show key
+         return mempty
        Just target -> return target
 
 note :: RSTParser Inlines
 note = try $ do
+  optional whitespace
   ref <- noteMarker
   char '_'
   state <- getState
   let notes = stateNotes state
   case lookup ref notes of
-    Nothing   -> fail "note not found"
+    Nothing   -> do
+      pos <- getPosition
+      addWarning (Just pos) $
+        "Could not find note for " ++ show ref
+      return mempty
     Just raw  -> do
       -- We temporarily empty the note list while parsing the note,
       -- so that we don't get infinite loops with notes inside notes...
