@@ -64,9 +64,11 @@ module Text.Pandoc.Shared (
                      compactify,
                      compactify',
                      compactify'DL,
+                     linesToPara,
                      Element (..),
                      hierarchicalize,
                      uniqueIdent,
+                     inlineListToIdentifier,
                      isHeaderBlock,
                      headerShift,
                      isTightList,
@@ -84,6 +86,7 @@ module Text.Pandoc.Shared (
                      fetchItem',
                      openURL,
                      collapseFilePath,
+                     filteredFilesFromArchive,
                      -- * Error handling
                      err,
                      warn,
@@ -110,6 +113,7 @@ import System.Exit (exitWith, ExitCode(..))
 import Data.Char ( toLower, isLower, isUpper, isAlpha,
                    isLetter, isDigit, isSpace )
 import Data.List ( find, stripPrefix, intercalate )
+import Data.Maybe (mapMaybe)
 import Data.Version ( showVersion )
 import qualified Data.Map as M
 import Network.URI ( escapeURIString, nonStrictRelativeTo,
@@ -123,6 +127,7 @@ import Text.Pandoc.MIME (MimeType, getMimeType)
 import System.FilePath ( (</>), takeExtension, dropExtension)
 import Data.Generics (Typeable, Data)
 import qualified Control.Monad.State as S
+import Control.Monad.Trans (MonadIO (..))
 import qualified Control.Exception as E
 import Control.Monad (msum, unless, MonadPlus(..))
 import Text.Pandoc.Pretty (charWidth)
@@ -132,7 +137,7 @@ import System.IO (stderr)
 import System.IO.Temp
 import Text.HTML.TagSoup (renderTagsOptions, RenderOptions(..), Tag(..),
          renderOptions)
-import Text.Pandoc.Compat.Monoid ((<>))
+import Data.Monoid ((<>))
 import qualified Data.ByteString as BS
 import qualified Data.ByteString.Char8 as B8
 import Data.ByteString.Base64 (decodeLenient)
@@ -152,16 +157,8 @@ import Paths_pandoc (getDataFileName)
 #ifdef HTTP_CLIENT
 import Network.HTTP.Client (httpLbs, responseBody, responseHeaders,
                             Request(port,host))
-#if MIN_VERSION_http_client(0,4,30)
 import Network.HTTP.Client (parseRequest)
-#else
-import Network.HTTP.Client (parseUrl)
-#endif
-#if MIN_VERSION_http_client(0,4,18)
 import Network.HTTP.Client (newManager)
-#else
-import Network.HTTP.Client (withManager)
-#endif
 import Network.HTTP.Client.Internal (addProxy)
 import Network.HTTP.Client.TLS (tlsManagerSettings)
 import System.Environment (getEnv)
@@ -639,6 +636,15 @@ compactify'DL items =
              | otherwise           -> items
            _                       -> items
 
+-- | Combine a list of lines by adding hard linebreaks.
+combineLines :: [[Inline]] -> [Inline]
+combineLines = intercalate [LineBreak]
+
+-- | Convert a list of lines into a paragraph with hard line breaks. This is
+--   useful e.g. for rudimentary support of LineBlock elements in writers.
+linesToPara :: [[Inline]] -> Block
+linesToPara = Para . combineLines
+
 isPara :: Block -> Bool
 isPara (Para _) = True
 isPara _        = False
@@ -873,7 +879,7 @@ readDefaultDataFile fname =
         go as     x    = x : as
 #else
   getDataFileName fname' >>= checkExistence >>= BS.readFile
-    where fname' = if fname == "README" then fname else "data" </> fname
+    where fname' = if fname == "MANUAL.txt" then fname else "data" </> fname
 
 checkExistence :: FilePath -> IO FilePath
 checkExistence fn = do
@@ -919,6 +925,10 @@ fetchItem sourceURL s =
           case parseURIReference' s' of
                Just u' -> openURL $ show $ u' `nonStrictRelativeTo` u
                Nothing -> openURL s' -- will throw error
+       (Nothing, s'@('/':'/':_)) ->  -- protocol-relative URI
+          case parseURIReference' s' of
+               Just u' -> openURL $ show $ u' `nonStrictRelativeTo` httpcolon
+               Nothing -> openURL s' -- will throw error
        (Nothing, s') ->
           case parseURI s' of  -- requires absolute URI
                -- We don't want to treat C:/ as a scheme:
@@ -929,6 +939,11 @@ fetchItem sourceURL s =
   where readLocalFile f = do
           cont <- BS.readFile f
           return (cont, mime)
+        httpcolon = URI{ uriScheme = "http:",
+                         uriAuthority = Nothing,
+                         uriPath = "",
+                         uriQuery = "",
+                         uriFragment = "" }
         dropFragmentAndQuery = takeWhile (\c -> c /= '?' && c /= '#')
         fp = unEscapeString $ dropFragmentAndQuery s
         mime = case takeExtension fp of
@@ -956,11 +971,7 @@ openURL u
     in  return $ Right (decodeLenient contents, Just mime)
 #ifdef HTTP_CLIENT
   | otherwise = withSocketsDo $ E.try $ do
-#if MIN_VERSION_http_client(0,4,30)
      let parseReq = parseRequest
-#else
-     let parseReq = parseUrl
-#endif
      (proxy :: Either E.SomeException String) <- E.try $ getEnv "http_proxy"
      req <- parseReq u
      req' <- case proxy of
@@ -968,16 +979,12 @@ openURL u
                      Right pr -> (parseReq pr >>= \r ->
                                   return $ addProxy (host r) (port r) req)
                                   `mplus` return req
-#if MIN_VERSION_http_client(0,4,18)
      resp <- newManager tlsManagerSettings >>= httpLbs req'
-#else
-     resp <- withManager tlsManagerSettings $ httpLbs req'
-#endif
      return (BS.concat $ toChunks $ responseBody resp,
              UTF8.toString `fmap` lookup hContentType (responseHeaders resp))
 #else
   | otherwise = E.try $ getBodyAndMimeType `fmap` browse
-              (do S.liftIO $ UTF8.hPutStrLn stderr $ "Fetching " ++ u ++ "..."
+              (do liftIO $ UTF8.hPutStrLn stderr $ "Fetching " ++ u ++ "..."
                   setOutHandler $ const (return ())
                   setAllowRedirects True
                   request (getRequest' u'))
@@ -1000,10 +1007,10 @@ err exitCode msg = do
   exitWith $ ExitFailure exitCode
   return undefined
 
-warn :: String -> IO ()
-warn msg = do
+warn :: MonadIO m => String -> m ()
+warn msg = liftIO $ do
   name <- getProgName
-  UTF8.hPutStrLn stderr $ name ++ ": " ++ msg
+  UTF8.hPutStrLn stderr $ "[" ++ name ++ " warning] " ++ msg
 
 mapLeft :: (a -> b) -> Either a c -> Either b c
 mapLeft f (Left x) = Left (f x)
@@ -1037,6 +1044,16 @@ collapseFilePath = Posix.joinPath . reverse . foldl go [] . splitDirectories
     isSingleton _ = Nothing
     checkPathSeperator = fmap isPathSeparator . isSingleton
 
+--
+-- File selection from the archive
+--
+filteredFilesFromArchive :: Archive -> (FilePath -> Bool) -> [(FilePath, BL.ByteString)]
+filteredFilesFromArchive zf f =
+  mapMaybe (fileAndBinary zf) (filter f (filesInArchive zf))
+  where
+    fileAndBinary :: Archive -> FilePath -> Maybe (FilePath, BL.ByteString)
+    fileAndBinary a fp = findEntryByPath fp a >>= \e -> Just (fp, fromEntry e)
+
 ---
 --- Squash blocks into inlines
 ---
@@ -1044,6 +1061,7 @@ collapseFilePath = Posix.joinPath . reverse . foldl go [] . splitDirectories
 blockToInlines :: Block -> [Inline]
 blockToInlines (Plain ils) = ils
 blockToInlines (Para ils) = ils
+blockToInlines (LineBlock lns) = combineLines lns
 blockToInlines (CodeBlock attr str) = [Code attr str]
 blockToInlines (RawBlock fmt str) = [RawInline fmt str]
 blockToInlines (BlockQuote blks) = blocksToInlines blks
@@ -1071,7 +1089,7 @@ blocksToInlinesWithSep sep blks = intercalate sep $ map blockToInlines blks
 
 blocksToInlines :: [Block] -> [Inline]
 blocksToInlines = blocksToInlinesWithSep [Space, Str "¶", Space]
-        
+
 
 --
 -- Safe read
