@@ -83,9 +83,11 @@ pandocToConTeXt options (Pandoc meta blocks) = do
                               ]
   let context =   defField "toc" (writerTableOfContents options)
                 $ defField "placelist" (intercalate ("," :: String) $
-                     take (writerTOCDepth options + if writerChapters options
-                                                       then 0
-                                                       else 1)
+                     take (writerTOCDepth options +
+                           case writerTopLevelDivision options of
+                             TopLevelPart    -> 0
+                             TopLevelChapter -> 0
+                             _               -> 1)
                        ["chapter","section","subsection","subsubsection",
                         "subsubsubsection","subsubsubsubsection"])
                 $ defField "body" main
@@ -96,9 +98,9 @@ pandocToConTeXt options (Pandoc meta blocks) = do
                     getField "lang" context)
                 $ defField "context-dir" (toContextDir $ getField "dir" context)
                 $ context
-  return $ if writerStandalone options
-              then renderTemplate' (writerTemplate options) context'
-              else main
+  return $ case writerTemplate options of
+                Nothing  -> main
+                Just tpl -> renderTemplate' tpl context'
 
 toContextDir :: Maybe String -> String
 toContextDir (Just "rtl") = "r2l"
@@ -163,6 +165,9 @@ blockToConTeXt (Para [Image attr txt (src,'f':'i':'g':':':_)]) = do
 blockToConTeXt (Para lst) = do
   contents <- inlineListToConTeXt lst
   return $ contents <> blankline
+blockToConTeXt (LineBlock lns) = do
+  doclines <- nowrap . vcat <$> mapM inlineListToConTeXt lns
+  return $ "\\startlines" $$ doclines $$ "\\stoplines" <> blankline
 blockToConTeXt (BlockQuote lst) = do
   contents <- blockListToConTeXt lst
   return $ "\\startblockquote" $$ nest 0 contents $$ "\\stopblockquote" <> blankline
@@ -409,7 +414,7 @@ inlineToConTeXt (Span (_,_,kvs) ils) = do
                        Nothing -> txt
   fmap (wrapLang . wrapDir) $ inlineListToConTeXt ils
 
--- | Craft the section header, inserting the secton reference, if supplied.
+-- | Craft the section header, inserting the section reference, if supplied.
 sectionHeader :: Attr
               -> Int
               -> [Inline]
@@ -418,21 +423,27 @@ sectionHeader (ident,classes,_) hdrLevel lst = do
   contents <- inlineListToConTeXt lst
   st <- get
   let opts = stOptions st
-  let level' = if writerChapters opts then hdrLevel - 1 else hdrLevel
+  let level' = case writerTopLevelDivision opts of
+                 TopLevelPart    -> hdrLevel - 2
+                 TopLevelChapter -> hdrLevel - 1
+                 TopLevelSection -> hdrLevel
+                 TopLevelDefault -> hdrLevel
   let ident' = toLabel ident
   let (section, chapter) = if "unnumbered" `elem` classes
                               then (text "subject", text "title")
                               else (text "section", text "chapter")
-  return $ if level' >= 1 && level' <= 5
-               then char '\\'
-                    <> text (concat (replicate (level' - 1) "sub"))
-                    <> section
-                    <> (if (not . null) ident' then brackets (text ident') else empty)
-                    <> braces contents
-                    <> blankline
-               else if level' == 0
-                       then char '\\' <> chapter <> braces contents
-                       else contents <> blankline
+  return $ case level' of
+             -1                   -> text "\\part" <> braces contents
+             0                    -> char '\\' <> chapter <> braces contents
+             n | n >= 1 && n <= 5 -> char '\\'
+                                     <> text (concat (replicate (n - 1) "sub"))
+                                     <> section
+                                     <> (if (not . null) ident'
+                                         then brackets (text ident')
+                                         else empty)
+                                     <> braces contents
+                                     <> blankline
+             _                    -> contents <> blankline
 
 fromBcp47' :: String -> String
 fromBcp47' = fromBcp47 . splitBy (=='-')
@@ -467,4 +478,3 @@ fromBcp47 x               = fromIso $ head x
     fromIso "vi" = "vn"
     fromIso "zh" = "cn"
     fromIso l    = l
-
