@@ -42,11 +42,10 @@ module Text.Pandoc.Writers.DokuWiki ( writeDokuWiki ) where
 import Text.Pandoc.Definition
 import Text.Pandoc.Options ( WriterOptions(
                                 writerTableOfContents
-                              , writerStandalone
                               , writerTemplate
                               , writerWrapText), WrapOption(..) )
-import Text.Pandoc.Shared ( escapeURI, removeFormatting, camelCaseToHyphenated
-                          , trimr, normalize, substitute  )
+import Text.Pandoc.Shared ( escapeURI, linesToPara, removeFormatting
+                          , camelCaseToHyphenated, trimr, normalize, substitute )
 import Text.Pandoc.Writers.Shared ( defField, metaToJSON )
 import Text.Pandoc.ImageSize
 import Text.Pandoc.Templates ( renderTemplate' )
@@ -102,9 +101,9 @@ pandocToDokuWiki opts (Pandoc meta blocks) = do
   let context = defField "body" main
                 $ defField "toc" (writerTableOfContents opts)
                 $ metadata
-  if writerStandalone opts
-     then return $ renderTemplate' (writerTemplate opts) context
-     else return main
+  case writerTemplate opts of
+       Nothing  -> return main
+       Just tpl -> return $ renderTemplate' tpl context
 
 -- | Escape special characters for DokuWiki.
 escapeString :: String -> String
@@ -147,6 +146,9 @@ blockToDokuWiki opts (Para inlines) = do
               then "<HTML><p></HTML>" ++ contents ++ "<HTML></p></HTML>"
               else contents ++ if null indent then "\n" else ""
 
+blockToDokuWiki opts (LineBlock lns) =
+  blockToDokuWiki opts $ linesToPara lns
+
 blockToDokuWiki _ (RawBlock f str)
   | f == Format "dokuwiki" = return str
   -- See https://www.dokuwiki.org/wiki:syntax
