@@ -57,8 +57,6 @@ import Text.Pandoc.Options ( WriterOptions(..)
                            , ObfuscationMethod(NoObfuscation) )
 import Text.Pandoc.Definition
 import Text.Pandoc.Walk (walk, walkM, query)
-import Data.Default
-import Text.Pandoc.Writers.Markdown (writePlain)
 import Control.Monad.State (modify, get, State, put, evalState)
 import Control.Monad (mplus, liftM, when)
 import Text.XML.Light ( unode, Element(..), unqual, Attr(..), add_attrs
@@ -228,10 +226,10 @@ addMetadataFromXML _ md = md
 
 metaValueToString :: MetaValue -> String
 metaValueToString (MetaString s) = s
-metaValueToString (MetaInlines ils) = writePlain def
-                                      (Pandoc nullMeta [Plain ils])
-metaValueToString (MetaBlocks bs) = writePlain def (Pandoc nullMeta bs)
-metaValueToString (MetaBool b) = show b
+metaValueToString (MetaInlines ils) = stringify ils
+metaValueToString (MetaBlocks bs) = stringify bs
+metaValueToString (MetaBool True) = "true"
+metaValueToString (MetaBool False) = "false"
 metaValueToString _ = ""
 
 getList :: String -> Meta -> (MetaValue -> a) -> [a]
@@ -344,7 +342,6 @@ writeEPUB opts doc@(Pandoc meta _) = do
            : ("css", "stylesheet.css")
            : writerVariables opts
   let opts' = opts{ writerEmailObfuscation = NoObfuscation
-                  , writerStandalone = True
                   , writerSectionDivs = True
                   , writerHtml5 = epub3
                   , writerVariables = vars
