@@ -1,5 +1,4 @@
 {-# LANGUAGE OverloadedStrings, CPP, ViewPatterns, ScopedTypeVariables #-}
-{-# OPTIONS_GHC -fno-warn-deprecations #-}
 {-
 Copyright (C) 2006-2015 John MacFarlane <jgm@berkeley.edu>
 
@@ -31,7 +30,7 @@ Conversion of 'Pandoc' documents to HTML.
 -}
 module Text.Pandoc.Writers.HTML ( writeHtml , writeHtmlString ) where
 import Text.Pandoc.Definition
-import Text.Pandoc.Compat.Monoid ((<>))
+import Data.Monoid ((<>))
 import Text.Pandoc.Shared
 import Text.Pandoc.Writers.Shared
 import Text.Pandoc.Options
@@ -62,7 +61,7 @@ import qualified Text.Blaze.Html5 as H5
 #endif
 import qualified Text.Blaze.XHtml1.Transitional as H
 import qualified Text.Blaze.XHtml1.Transitional.Attributes as A
-import Text.Blaze.Renderer.String (renderHtml)
+import Text.Blaze.Html.Renderer.String (renderHtml)
 import Text.TeXMath
 import Text.XML.Light.Output
 import Text.XML.Light (unode, elChildren, unqual)
@@ -103,17 +102,19 @@ nl opts = if writerWrapText opts == WrapNone
 writeHtmlString :: WriterOptions -> Pandoc -> String
 writeHtmlString opts d =
   let (body, context) = evalState (pandocToHtml opts d) defaultWriterState
-  in  if writerStandalone opts
-         then inTemplate opts context body
-         else renderHtml body
+  in  case writerTemplate opts of
+           Nothing  -> renderHtml body
+           Just tpl -> renderTemplate' tpl $
+                         defField "body" (renderHtml body) context
 
 -- | Convert Pandoc document to Html structure.
 writeHtml :: WriterOptions -> Pandoc -> Html
 writeHtml opts d =
   let (body, context) = evalState (pandocToHtml opts d) defaultWriterState
-  in  if writerStandalone opts
-         then inTemplate opts context body
-         else body
+  in  case writerTemplate opts of
+           Nothing  -> body
+           Just tpl -> renderTemplate' tpl $
+                         defField "body" (renderHtml body) context
 
 -- result is (title, authors, date, toc, body, new variables)
 pandocToHtml :: WriterOptions
@@ -195,14 +196,6 @@ pandocToHtml opts (Pandoc meta blocks) = do
                   metadata
   return (thebody, context)
 
-inTemplate :: TemplateTarget a
-           => WriterOptions
-           -> Value
-           -> Html
-           -> a
-inTemplate opts context body = renderTemplate' (writerTemplate opts)
-                             $ defField "body" (renderHtml body) context
-
 -- | Like Text.XHtml's identifier, but adds the writerIdentifierPrefix
 prefixedId :: WriterOptions -> String -> Attribute
 prefixedId opts s =
@@ -278,15 +271,13 @@ elementToHtml slideLevel opts (Sec level num (id',classes,keyvals) title' elemen
   let slide = writerSlideVariant opts /= NoSlides && level <= slideLevel
   let num' = zipWith (+) num (writerNumberOffset opts ++ repeat 0)
   modify $ \st -> st{stSecNum = num'}  -- update section number
-  -- always use level 1 for slide titles
-  let level' = if slide then 1 else level
   let titleSlide = slide && level < slideLevel
   header' <- if title' == [Str "\0"]  -- marker for hrule
                 then return mempty
                 else do
                   modify (\st -> st{ stElement = True})
                   res <- blockToHtml opts
-                           (Header level' (id',classes,keyvals) title')
+                           (Header level (id',classes,keyvals) title')
                   modify (\st -> st{ stElement = False})
                   return res
 
@@ -460,9 +451,21 @@ blockToHtml opts (Para [Image attr txt (s,'f':'i':'g':':':tit)]) = do
                     [nl opts, img, capt, nl opts]
               else H.div ! A.class_ "figure" $ mconcat
                     [nl opts, img, nl opts, capt, nl opts]
-blockToHtml opts (Para lst) = do
-  contents <- inlineListToHtml opts lst
-  return $ H.p contents
+blockToHtml opts (Para lst)
+  | isEmptyRaw lst = return mempty
+  | otherwise = do
+      contents <- inlineListToHtml opts lst
+      return $ H.p contents
+  where
+    isEmptyRaw [RawInline f _] = f /= (Format "html")
+    isEmptyRaw _                 = False
+blockToHtml opts (LineBlock lns) =
+  if writerWrapText opts == WrapNone
+  then blockToHtml opts $ linesToPara lns
+  else do
+    let lf = preEscapedString "\n"
+    htmlLines <- mconcat . intersperse lf <$> mapM (inlineListToHtml opts) lns
+    return $ H.div ! A.style "white-space: pre-line;" $ htmlLines
 blockToHtml opts (Div attr@(ident, classes, kvs) bs) = do
   let speakerNotes = "notes" `elem` classes
   -- we don't want incremental output inside speaker notes, see #1394
@@ -807,7 +810,7 @@ inlineToHtml opts inline =
               let brtag = if writerHtml5 opts then H5.br else H.br
               return  $ case t of
                          InlineMath  -> m
-                         DisplayMath -> brtag >> m >> brtag 
+                         DisplayMath -> brtag >> m >> brtag
     (RawInline f str)
       | f == Format "html" -> return $ preEscapedString str
       | otherwise          -> return mempty
