@@ -2,7 +2,7 @@
 
 {- |
    Module      : Text.Pandoc.Writers.ICML
-   Copyright   : Copyright (C) 2013 github.com/mb21
+   Copyright   : Copyright (C) 2013-2016 github.com/mb21
    License     : GNU GPL, version 2 or above
 
    Stability   : alpha
@@ -18,7 +18,7 @@ import Text.Pandoc.Definition
 import Text.Pandoc.XML
 import Text.Pandoc.Readers.TeXMath (texMathToInlines)
 import Text.Pandoc.Writers.Shared
-import Text.Pandoc.Shared (splitBy, fetchItem, warn)
+import Text.Pandoc.Shared (linesToPara, splitBy, fetchItem, warn)
 import Text.Pandoc.Options
 import Text.Pandoc.Templates (renderTemplate')
 import Text.Pandoc.Pretty
@@ -141,9 +141,9 @@ writeICML opts (Pandoc meta blocks) = do
               $ defField "parStyles"  (render' $ parStylesToDoc st)
               $ defField "hyperlinks" (render' $ hyperlinksToDoc $ links st)
               $ metadata
-  return $ if writerStandalone opts
-              then renderTemplate' (writerTemplate opts) context
-              else main
+  return $ case writerTemplate opts of
+                Nothing  -> main
+                Just tpl -> renderTemplate' tpl context
 
 -- | Auxilary functions for parStylesToDoc and charStylesToDoc.
 contains :: String -> (String, (String, String)) -> [(String, String)]
@@ -297,6 +297,8 @@ blockToICML opts style (Para img@[Image _ txt (_,'f':'i':'g':':':_)]) = do
   caption <- parStyle opts (imgCaptionName:style) txt
   return $ intersperseBrs [figure, caption]
 blockToICML opts style (Para lst) = parStyle opts (paragraphName:style) lst
+blockToICML opts style (LineBlock lns) =
+  blockToICML opts style $ linesToPara lns
 blockToICML opts style (CodeBlock _ str) = parStyle opts (codeBlockName:style) $ [Str str]
 blockToICML _ _ (RawBlock f str)
   | f == Format "icml" = return $ text str
@@ -321,13 +323,13 @@ blockToICML opts style (Table caption aligns widths headers rows) =
                   else length $ head rows
       rowsToICML [] _ = return empty
       rowsToICML (col:rest) rowNr =
-        liftM2 ($$) (colsToICML col rowNr (0::Int)) $ rowsToICML rest (rowNr+1)
-      colsToICML [] _ _ = return empty
-      colsToICML (cell:rest) rowNr colNr = do
+        liftM2 ($$) (colsToICML col aligns rowNr (0::Int)) $ rowsToICML rest (rowNr+1)
+      colsToICML [] _ _ _ = return empty
+      colsToICML _ [] _ _ = return empty
+      colsToICML (cell:rest) (alig:restAligns) rowNr colNr = do
         let stl  = if rowNr == 0 && not noHeader
                       then tableHeaderName:style'
                       else style'
-            alig = aligns !! colNr
             stl' | alig == AlignLeft = alignLeftName : stl
                  | alig == AlignRight = alignRightName : stl
                  | alig == AlignCenter = alignCenterName : stl
@@ -335,7 +337,7 @@ blockToICML opts style (Table caption aligns widths headers rows) =
         c <- blocksToICML opts stl' cell
         let cl = return $ inTags True "Cell"
                    [("Name", show colNr ++":"++ show rowNr), ("AppliedCellStyle","CellStyle/Cell")] c
-        liftM2 ($$) cl $ colsToICML rest rowNr (colNr+1)
+        liftM2 ($$) cl $ colsToICML rest restAligns rowNr (colNr+1)
   in  do
       let tabl = if noHeader
                     then rows
@@ -532,13 +534,13 @@ imageICML opts style attr (src, _) = do
   res  <- liftIO $ fetchItem (writerSourceURL opts) src
   imgS <- case res of
             Left (_) -> do
-              liftIO $ warn $ "Could not find image `" ++ src ++ "', skipping..."
+              warn $ "Could not find image `" ++ src ++ "', skipping..."
               return def
             Right (img, _) -> do
               case imageSize img of
                 Right size -> return size
                 Left msg   -> do
-                  return $ warn $ "Could not determine image size in `" ++
+                  warn $ "Could not determine image size in `" ++
                     src ++ "': " ++ msg
                   return def
   let (ow, oh) = sizeInPoints imgS
