@@ -87,7 +87,11 @@ writeLaTeX options document =
                 stOptions = options, stVerbInNote = False,
                 stTable = False, stStrikeout = False,
                 stUrl = False, stGraphics = False,
-                stLHS = False, stBook = writerChapters options,
+                stLHS = False,
+                stBook = (case writerTopLevelDivision options of
+                           TopLevelPart    -> True
+                           TopLevelChapter -> True
+                           _               -> False),
                 stCsquotes = False, stHighlighting = False,
                 stIncremental = writerIncremental options,
                 stInternalLinks = [], stUsesEuro = False }
@@ -105,7 +109,7 @@ pandocToLaTeX options (Pandoc meta blocks) = do
   let isInternalLink (Link _ _ ('#':xs,_))  = [xs]
       isInternalLink _                      = []
   modify $ \s -> s{ stInternalLinks = query isInternalLink blocks' }
-  let template = writerTemplate options
+  let template = maybe "" id $ writerTemplate options
   -- set stBook depending on documentclass
   let colwidth = if writerWrapText options == WrapAuto
                     then Just $ writerColumns options
@@ -242,9 +246,9 @@ pandocToLaTeX options (Pandoc meta blocks) = do
                                       Just "rtl" -> True
                                       _          -> False)
         $ context
-  return $ if writerStandalone options
-              then renderTemplate' template context'
-              else main
+  return $ case writerTemplate options of
+                Nothing  -> main
+                Just tpl -> renderTemplate' tpl context'
 
 -- | Convert Elements to LaTeX
 elementToLaTeX :: WriterOptions -> Element -> State WriterState Doc
@@ -276,6 +280,7 @@ stringToLaTeX  ctx (x:xs) = do
        '€' -> "\\euro{}" ++ rest
        '{' -> "\\{" ++ rest
        '}' -> "\\}" ++ rest
+       '`' | ctx == CodeString -> "\\textasciigrave{}" ++ rest
        '$' | not isUrl -> "\\$" ++ rest
        '%' -> "\\%" ++ rest
        '&' -> "\\&" ++ rest
@@ -296,6 +301,7 @@ stringToLaTeX  ctx (x:xs) = do
        ']' -> "{]}" ++ rest  -- optional arguments
        '\'' | ctx == CodeString -> "\\textquotesingle{}" ++ rest
        '\160' -> "~" ++ rest
+       '\x202F' -> "\\," ++ rest
        '\x2026' -> "\\ldots{}" ++ rest
        '\x2018' | ligatures -> "`" ++ rest
        '\x2019' | ligatures -> "'" ++ rest
@@ -421,7 +427,7 @@ blockToLaTeX (Para [Image attr@(ident, _, _) txt (src,'f':'i':'g':':':tit)]) = d
   lab <- labelFor ident
   let caption = "\\caption" <> captForLof <> braces capt <> lab
   figure <- hypertarget ident (cr <>
-            "\\begin{figure}[htbp]" $$ "\\centering" $$ img $$
+            "\\begin{figure}" $$ "\\centering" $$ img $$
             caption $$ "\\end{figure}" <> cr)
   return $ if inNote
               -- can't have figures in notes
@@ -435,6 +441,8 @@ blockToLaTeX (Para [Str ".",Space,Str ".",Space,Str "."]) = do
      else inlineListToLaTeX [Str ".",Space,Str ".",Space,Str "."]
 blockToLaTeX (Para lst) =
   inlineListToLaTeX $ dropWhile isLineBreakOrSpace lst
+blockToLaTeX (LineBlock lns) = do
+  blockToLaTeX $ linesToPara lns
 blockToLaTeX (BlockQuote lst) = do
   beamer <- writerBeamer `fmap` gets stOptions
   case lst of
@@ -500,7 +508,7 @@ blockToLaTeX (CodeBlock (identifier,classes,keyvalAttr) str) = do
         case highlight formatLaTeXBlock ("",classes,keyvalAttr) str of
                Nothing -> rawCodeBlock
                Just  h -> modify (\st -> st{ stHighlighting = True }) >>
-                          return (flush $ linkAnchor $$ text h)
+                          return (flush $ linkAnchor $$ text (T.unpack h))
   case () of
      _ | isEnabled Ext_literate_haskell opts && "haskell" `elem` classes &&
          "literate" `elem` classes                      -> lhsCodeBlock
@@ -630,7 +638,15 @@ tableRowToLaTeX header aligns widths cols = do
   -- scale factor compensates for extra space between columns
   -- so the whole table isn't larger than columnwidth
   let scaleFactor = 0.97 ** fromIntegral (length aligns)
-  let widths' = map (scaleFactor *) widths
+  let isSimple [Plain _] = True
+      isSimple [Para  _] = True
+      isSimple []        = True
+      isSimple _         = False
+  -- simple tables have to have simple cells:
+  let widths' = if not (all isSimple cols)
+                   then replicate (length aligns)
+                          (0.97 / fromIntegral (length aligns))
+                   else map (scaleFactor *) widths
   cells <- mapM (tableCellToLaTeX header) $ zip3 widths' aligns cols
   return $ hsep (intersperse "&" cells) <> "\\tabularnewline"
 
@@ -728,15 +744,16 @@ sectionHeader :: Bool    -- True for unnumbered
 sectionHeader unnumbered ident level lst = do
   txt <- inlineListToLaTeX lst
   plain <- stringToLaTeX TextString $ concatMap stringify lst
-  let noNote (Note _) = Str ""
-      noNote x        = x
-  let lstNoNotes = walk noNote lst
+  let removeInvalidInline (Note _)      = []
+      removeInvalidInline (Span (id', _, _) _) | not (null id') = []
+      removeInvalidInline (Image _ _ _) = []
+      removeInvalidInline x             = [x]
+  let lstNoNotes = foldr (mappend . (\x -> walkM removeInvalidInline x)) mempty lst
   txtNoNotes <- inlineListToLaTeX lstNoNotes
-  let star = if unnumbered then text "*" else empty
   -- footnotes in sections don't work (except for starred variants)
   -- unless you specify an optional argument:
   -- \section[mysec]{mysec\footnote{blah}}
-  optional <- if unnumbered || lstNoNotes == lst
+  optional <- if unnumbered || lstNoNotes == lst || lstNoNotes == []
                  then return empty
                  else do
                    return $ brackets txtNoNotes
@@ -745,13 +762,23 @@ sectionHeader unnumbered ident level lst = do
                     else braces (text "\\texorpdfstring"
                          <> braces txt
                          <> braces (text plain))
-  let stuffing = star <> optional <> contents
   book <- gets stBook
   opts <- gets stOptions
-  let level' = if book || writerChapters opts then level - 1 else level
+  let topLevelDivision = if book && writerTopLevelDivision opts == TopLevelDefault
+                         then TopLevelChapter
+                         else writerTopLevelDivision opts
+  let level' = if writerBeamer opts &&
+                  topLevelDivision `elem` [TopLevelPart, TopLevelChapter]
+               -- beamer has parts but no chapters
+               then if level == 1 then -1 else level - 1
+               else case topLevelDivision of
+                      TopLevelPart    -> level - 2
+                      TopLevelChapter -> level - 1
+                      TopLevelSection -> level
+                      TopLevelDefault -> level
   let sectionType = case level' of
-                          0  | writerBeamer opts -> "part"
-                             | otherwise -> "chapter"
+                          -1 -> "part"
+                          0  -> "chapter"
                           1  -> "section"
                           2  -> "subsection"
                           3  -> "subsubsection"
@@ -765,6 +792,8 @@ sectionHeader unnumbered ident level lst = do
                   -- see http://tex.stackexchange.com/questions/169830/
                   else empty
   lab <- labelFor ident
+  let star = if unnumbered && level' < 4 then text "*" else empty
+  let stuffing = star <> optional <> contents
   stuffing' <- hypertarget ident $ text ('\\':sectionType) <> stuffing <> lab
   return $ if level' > 5
               then txt
@@ -887,7 +916,7 @@ inlineToLaTeX (Code (_,classes,_) str) = do
            case highlight formatLaTeXInline ("",classes,[]) str of
                   Nothing -> rawCode
                   Just  h -> modify (\st -> st{ stHighlighting = True }) >>
-                             return (text h)
+                             return (text (T.unpack h))
          rawCode = liftM (text . (\s -> "\\texttt{" ++ escapeSpaces s ++ "}"))
                           $ stringToLaTeX CodeString str
            where
@@ -1091,8 +1120,9 @@ citationsToBiblatex (c:cs) = do
   return $ text cmd <> foldl' (<>) empty args
     where
        cmd = case citationMode c of
-                  AuthorInText -> "\\textcites"
-                  _            -> "\\autocites"
+                  SuppressAuthor -> "\\autocites*"
+                  AuthorInText   -> "\\textcites"
+                  NormalCitation -> "\\autocites"
        convertOne Citation { citationId = k
                            , citationPrefix = p
                            , citationSuffix = s
