@@ -41,7 +41,6 @@ import Data.Maybe (fromMaybe)
 import Text.Pandoc.Pretty
 import Text.Pandoc.Builder (deleteMeta)
 import Control.Monad.State
-import Data.Char ( isDigit )
 
 type Notes = [[Block]]
 data WriterState = WriterState { stNotes  :: Notes
@@ -62,10 +61,11 @@ pandocToMan opts (Pandoc meta blocks) = do
   let title' = render' titleText
   let setFieldsFromTitle =
        case break (== ' ') title' of
-           (cmdName, rest) -> case reverse cmdName of
-                                   (')':d:'(':xs) | isDigit d ->
-                                     defField "title" (reverse xs) .
-                                     defField "section" [d] .
+           (cmdName, rest) -> case break (=='(') cmdName of
+                                   (xs, '(':ys) | not (null ys) &&
+                                                  last ys == ')' ->
+                                     defField "title" xs .
+                                     defField "section" (init ys) .
                                      case splitBy (=='|') rest of
                                           (ft:hds) ->
                                             defField "footer" (trim ft) .
@@ -88,9 +88,9 @@ pandocToMan opts (Pandoc meta blocks) = do
               $ defField "hyphenate" True
               $ defField "pandoc-version" pandocVersion
               $ metadata
-  if writerStandalone opts
-     then return $ renderTemplate' (writerTemplate opts) context
-     else return main
+  case writerTemplate opts of
+       Nothing  -> return main
+       Just tpl -> return $ renderTemplate' tpl context
 
 -- | Return man representation of notes.
 notesToMan :: WriterOptions -> [[Block]] -> State WriterState Doc
@@ -171,6 +171,8 @@ blockToMan opts (Para inlines) = do
   contents <- liftM vcat $ mapM (inlineListToMan opts) $
     splitSentences inlines
   return $ text ".PP" $$ contents
+blockToMan opts (LineBlock lns) =
+  blockToMan opts $ linesToPara lns
 blockToMan _ (RawBlock f str)
   | f == Format "man" = return $ text str
   | otherwise         = return empty
@@ -298,10 +300,6 @@ blockListToMan opts blocks =
 
 -- | Convert list of Pandoc inline elements to man.
 inlineListToMan :: WriterOptions -> [Inline] -> State WriterState Doc
--- if list starts with ., insert a zero-width character \& so it
--- won't be interpreted as markup if it falls at the beginning of a line.
-inlineListToMan opts lst@(Str ('.':_) : _) = mapM (inlineToMan opts) lst >>=
-  (return . (text "\\&" <>)  . hcat)
 inlineListToMan opts lst = mapM (inlineToMan opts) lst >>= (return . hcat)
 
 -- | Convert Pandoc inline element to man.
@@ -333,6 +331,8 @@ inlineToMan opts (Cite _ lst) =
   inlineListToMan opts lst
 inlineToMan _ (Code _ str) =
   return $ text $ "\\f[C]" ++ escapeCode str ++ "\\f[]"
+inlineToMan _ (Str str@('.':_)) =
+  return $ afterBreak "\\&" <> text (escapeString str)
 inlineToMan _ (Str str) = return $ text $ escapeString str
 inlineToMan opts (Math InlineMath str) =
   inlineListToMan opts $ texMathToInlines InlineMath str
@@ -367,4 +367,3 @@ inlineToMan _ (Note contents) = do
   notes <- liftM stNotes get
   let ref = show $ (length notes)
   return $ char '[' <> text ref <> char ']'
-
