@@ -1,6 +1,6 @@
 {-# LANGUAGE ScopedTypeVariables #-}
 {-
-Copyright (C) 2008-2014 John MacFarlane <jgm@berkeley.edu>
+Copyright (C) 2008-2015 John MacFarlane <jgm@berkeley.edu>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 {- |
    Module      : Text.Pandoc.Writers.ODT
-   Copyright   : Copyright (C) 2008-2014 John MacFarlane
+   Copyright   : Copyright (C) 2008-2015 John MacFarlane
    License     : GNU GPL, version 2 or above
 
    Maintainer  : John MacFarlane <jgm@berkeley.edu>
@@ -37,10 +37,10 @@ import Text.TeXMath
 import qualified Data.ByteString.Lazy as B
 import Text.Pandoc.UTF8 ( fromStringLazy )
 import Codec.Archive.Zip
-import Control.Applicative ((<$>))
-import Text.Pandoc.Options ( WriterOptions(..) )
-import Text.Pandoc.Shared ( stringify, readDataFile, fetchItem', warn )
-import Text.Pandoc.ImageSize ( imageSize, sizeInPoints )
+import Text.Pandoc.Options ( WriterOptions(..), WrapOption(..) )
+import Text.Pandoc.Shared ( stringify, fetchItem', warn,
+                            getDefaultReferenceODT )
+import Text.Pandoc.ImageSize
 import Text.Pandoc.MIME ( getMimeType, extensionFromMimeType )
 import Text.Pandoc.Definition
 import Text.Pandoc.Walk
@@ -60,15 +60,14 @@ writeODT :: WriterOptions  -- ^ Writer options
 writeODT opts doc@(Pandoc meta _) = do
   let datadir = writerUserDataDir opts
   let title = docTitle meta
-  refArchive <- liftM toArchive $
+  refArchive <-
        case writerReferenceODT opts of
-             Just f -> B.readFile f
-             Nothing -> (B.fromChunks . (:[])) `fmap`
-                           readDataFile datadir "reference.odt"
+             Just f -> liftM toArchive $ B.readFile f
+             Nothing -> getDefaultReferenceODT datadir
   -- handle formulas and pictures
   picEntriesRef <- newIORef ([] :: [Entry])
   doc' <- walkM (transformPicMath opts picEntriesRef) $ walk fixDisplayMath doc
-  let newContents = writeOpenDocument opts{writerWrapText = False} doc'
+  let newContents = writeOpenDocument opts{writerWrapText = WrapNone} doc'
   epochtime <- floor `fmap` getPOSIXTime
   let contentEntry = toEntry "content.xml" epochtime
                      $ fromStringLazy newContents
@@ -126,17 +125,36 @@ writeODT opts doc@(Pandoc meta _) = do
                   $ addEntryToArchive metaEntry archive'
   return $ fromArchive archive''
 
+-- | transform both Image and Math elements
 transformPicMath :: WriterOptions -> IORef [Entry] -> Inline -> IO Inline
-transformPicMath opts entriesRef (Image lab (src,_)) = do
+transformPicMath opts entriesRef (Image attr@(id', cls, _) lab (src,t)) = do
   res <- fetchItem' (writerMediaBag opts) (writerSourceURL opts) src
   case res of
      Left (_ :: E.SomeException) -> do
        warn $ "Could not find image `" ++ src ++ "', skipping..."
        return $ Emph lab
      Right (img, mbMimeType) -> do
-       let size = imageSize img
-       let (w,h) = fromMaybe (0,0) $ sizeInPoints `fmap` size
-       let tit' = show w ++ "x" ++ show h
+       (ptX, ptY) <- case imageSize img of
+                       Right s  -> return $ sizeInPoints s
+                       Left msg -> do
+                         warn $ "Could not determine image size in `" ++
+                           src ++ "': " ++ msg
+                         return (100, 100)
+       let dims =
+             case (getDim Width, getDim Height) of
+               (Just w, Just h)              -> [("width", show w), ("height", show h)]
+               (Just w@(Percent _), Nothing) -> [("width", show w), ("style:rel-height", "scale")]
+               (Nothing, Just h@(Percent _)) -> [("style:rel-width", "scale"), ("height", show h)]
+               (Just w@(Inch i), Nothing)    -> [("width", show w), ("height", show (i / ratio) ++ "in")]
+               (Nothing, Just h@(Inch i))    -> [("width", show (i * ratio) ++ "in"), ("height", show h)]
+               _                             -> [("width", show ptX ++ "pt"), ("height", show ptY ++ "pt")]
+             where
+               ratio = ptX / ptY
+               getDim dir = case (dimension dir attr) of
+                              Just (Percent i) -> Just $ Percent i
+                              Just dim         -> Just $ Inch $ inInch opts dim
+                              Nothing          -> Nothing
+       let  newattr = (id', cls, dims)
        entries <- readIORef entriesRef
        let extension = fromMaybe (takeExtension $ takeWhile (/='?') src)
                            (mbMimeType >>= extensionFromMimeType)
@@ -145,7 +163,7 @@ transformPicMath opts entriesRef (Image lab (src,_)) = do
        epochtime <- floor `fmap` getPOSIXTime
        let entry = toEntry newsrc epochtime $ toLazy img
        modifyIORef entriesRef (entry:)
-       return $ Image lab (newsrc, tit')
+       return $ Image newattr lab (newsrc, t)
 transformPicMath _ entriesRef (Math t math) = do
   entries <- readIORef entriesRef
   let dt = if t == InlineMath then DisplayInline else DisplayBlock
