@@ -33,6 +33,7 @@ module Text.Pandoc.Writers.OpenDocument ( writeOpenDocument ) where
 import Text.Pandoc.Definition
 import Text.Pandoc.Options
 import Text.Pandoc.XML
+import Text.Pandoc.Shared (linesToPara)
 import Text.Pandoc.Templates (renderTemplate')
 import Text.Pandoc.Readers.TeXMath
 import Text.Pandoc.Pretty
@@ -40,8 +41,11 @@ import Text.Printf ( printf )
 import Control.Arrow ( (***), (>>>) )
 import Control.Monad.State hiding ( when )
 import Data.Char (chr)
+import qualified Data.Set as Set
 import qualified Data.Map as Map
 import Text.Pandoc.Writers.Shared
+import Data.List (sortBy)
+import Data.Ord (comparing)
 
 -- | Auxiliary function to convert Plain block to Para.
 plainToPara :: Block -> Block
@@ -57,8 +61,8 @@ data WriterState =
                 , stTableStyles   :: [Doc]
                 , stParaStyles    :: [Doc]
                 , stListStyles    :: [(Int, [Doc])]
-                , stTextStyles    :: [Doc]
-                , stTextStyleAttr :: Map.Map TextStyle [(String,String)]
+                , stTextStyles    :: Map.Map (Set.Set TextStyle) (String, Doc)
+                , stTextStyleAttr :: Set.Set TextStyle
                 , stIndentPara    :: Int
                 , stInDefinition  :: Bool
                 , stTight         :: Bool
@@ -72,8 +76,8 @@ defaultWriterState =
                 , stTableStyles   = []
                 , stParaStyles    = []
                 , stListStyles    = []
-                , stTextStyles    = []
-                , stTextStyleAttr = Map.empty
+                , stTextStyles    = Map.empty
+                , stTextStyleAttr = Set.empty
                 , stIndentPara    = 0
                 , stInDefinition  = False
                 , stTight         = False
@@ -93,14 +97,13 @@ addNote i = modify $ \s -> s { stNotes = i : stNotes s }
 addParaStyle :: Doc -> State WriterState ()
 addParaStyle i = modify $ \s -> s { stParaStyles = i : stParaStyles s }
 
-addTextStyle :: Doc -> State WriterState ()
-addTextStyle i = modify $ \s -> s { stTextStyles = i : stTextStyles s }
+addTextStyle :: Set.Set TextStyle -> (String, Doc) -> State WriterState ()
+addTextStyle attrs i = modify $ \s ->
+  s { stTextStyles = Map.insert attrs i (stTextStyles s) }
 
-addTextStyleAttr :: (TextStyle, [(String,String)]) -> State WriterState ()
-addTextStyleAttr (ts, xs) = modify $ \s -> s { stTextStyleAttr = Map.insert ts xs (stTextStyleAttr s) }
-
-rmTextStyleAttr :: TextStyle -> State WriterState ()
-rmTextStyleAttr ts = modify $ \s -> s { stTextStyleAttr = Map.delete ts (stTextStyleAttr s) }
+addTextStyleAttr :: TextStyle -> State WriterState ()
+addTextStyleAttr t = modify $ \s ->
+  s { stTextStyleAttr = Set.insert t (stTextStyleAttr s) }
 
 increaseIndent :: State WriterState ()
 increaseIndent = modify $ \s -> s { stIndentPara = 1 + stIndentPara s }
@@ -135,20 +138,33 @@ inSpanTags :: String -> Doc -> Doc
 inSpanTags s = inTags False "text:span" [("text:style-name",s)]
 
 withTextStyle :: TextStyle -> State WriterState a -> State WriterState a
-withTextStyle s f = addTextStyleAttr (s,textStyleAttr s) >>
-                    f >>= \r -> rmTextStyleAttr s >> return r
+withTextStyle s f = do
+  oldTextStyleAttr <- gets stTextStyleAttr
+  addTextStyleAttr s
+  res <- f
+  modify $ \st -> st{ stTextStyleAttr = oldTextStyleAttr }
+  return res
 
 inTextStyle :: Doc -> State WriterState Doc
 inTextStyle d = do
   at <- gets stTextStyleAttr
-  if Map.null at
+  if Set.null at
      then return d
      else do
-       tn <- (+) 1 . length  <$> gets stTextStyles
-       addTextStyle $ inTags False "style:style" [("style:name"  , "T" ++ show tn)
-                                                 ,("style:family", "text"        )]
-                    $ selfClosingTag "style:text-properties" (concatMap snd $ Map.toList at)
-       return $ inTags False "text:span" [("text:style-name","T" ++ show tn)] d
+       styles <- gets stTextStyles
+       case Map.lookup at styles of
+            Just (styleName, _) -> return $
+              inTags False "text:span" [("text:style-name",styleName)] d
+            Nothing -> do
+              let styleName = "T" ++ show (Map.size styles + 1)
+              addTextStyle at (styleName,
+                     inTags False "style:style"
+                       [("style:name", styleName)
+                       ,("style:family", "text")]
+                    $ selfClosingTag "style:text-properties"
+                       (concatMap textStyleAttr (Set.toList at)))
+              return $ inTags False
+                  "text:span" [("text:style-name",styleName)] d
 
 inHeaderTags :: Int -> Doc -> State WriterState Doc
 inHeaderTags i d =
@@ -187,7 +203,9 @@ writeOpenDocument opts (Pandoc meta blocks) =
                   meta
            b <- render' `fmap` blocksToOpenDocument opts blocks
            return (b, m)
-      styles   = stTableStyles s ++ stParaStyles s ++ stTextStyles s
+      styles   = stTableStyles s ++ stParaStyles s ++
+                     map snd (reverse $ sortBy (comparing fst) $
+                        Map.elems (stTextStyles s))
       listStyle (n,l) = inTags True "text:list-style"
                           [("style:name", "L" ++ show n)] (vcat l)
       listStyles  = map listStyle (stListStyles s)
@@ -195,9 +213,9 @@ writeOpenDocument opts (Pandoc meta blocks) =
       context = defField "body" body
               $ defField "automatic-styles" (render' automaticStyles)
               $ metadata
-  in  if writerStandalone opts
-         then renderTemplate' (writerTemplate opts) context
-         else body
+  in  case writerTemplate opts of
+           Nothing  -> body
+           Just tpl -> renderTemplate' tpl context
 
 withParagraphStyle :: WriterOptions -> String -> [Block] -> State WriterState Doc
 withParagraphStyle  o s (b:bs)
@@ -208,7 +226,7 @@ withParagraphStyle _ _ [] = return empty
 
 inPreformattedTags :: String -> State WriterState Doc
 inPreformattedTags s = do
-  n <- paraStyle "Preformatted_20_Text" []
+  n <- paraStyle [("style:parent-style-name","Preformatted_20_Text")]
   return . inParagraphTagsWithStyle ("P" ++ show n) . handleSpaces $ s
 
 orderedListToOpenDocument :: WriterOptions -> Int -> [[Block]] -> State WriterState Doc
@@ -269,7 +287,8 @@ deflistItemToOpenDocument o (t,d) = do
 inBlockQuote :: WriterOptions -> Int -> [Block] -> State WriterState Doc
 inBlockQuote  o i (b:bs)
     | BlockQuote l <- b = do increaseIndent
-                             ni <- paraStyle "Quotations" []
+                             ni <- paraStyle
+                                   [("style:parent-style-name","Quotations")]
                              go =<< inBlockQuote o ni (map plainToPara l)
     | Para       l <- b = do go =<< inParagraphTagsWithStyle ("P" ++ show  i) <$> inlinesToOpenDocument o l
     | otherwise         = do go =<< blockToOpenDocument o b
@@ -291,6 +310,7 @@ blockToOpenDocument o bs
     | Para           b <- bs = if null b
                                   then return empty
                                   else inParagraphTags =<< inlinesToOpenDocument o b
+    | LineBlock      b <- bs = blockToOpenDocument o $ linesToPara b
     | Div _ xs         <- bs = blocksToOpenDocument o xs
     | Header     i _ b <- bs = setFirstPara >>
                                (inHeaderTags  i =<< inlinesToOpenDocument o b)
@@ -314,7 +334,8 @@ blockToOpenDocument o bs
                            return r
       preformatted  s = (flush . vcat) <$> mapM (inPreformattedTags . escapeStringForXML) (lines s)
       mkBlockQuote  b = do increaseIndent
-                           i <- paraStyle "Quotations" []
+                           i <- paraStyle
+                                 [("style:parent-style-name","Quotations")]
                            inBlockQuote o i (map plainToPara b)
       orderedList a b = do (ln,pn) <- newOrderedListStyle (isTightList b) a
                            inTags True "text:list" [ ("text:style-name", "L" ++ show ln)]
@@ -334,7 +355,7 @@ blockToOpenDocument o bs
         mapM_ addParaStyle . newPara $ paraHStyles ++ paraStyles
         captionDoc <- if null c
                       then return empty
-                      else withParagraphStyle o "TableCaption" [Para c]
+                      else withParagraphStyle o "Table" [Para c]
         th <- if all null h
                  then return empty
                  else colHeadsToOpenDocument o name (map fst paraHStyles) h
@@ -369,20 +390,40 @@ tableItemToOpenDocument o tn (n,i) =
 
 -- | Convert a list of inline elements to OpenDocument.
 inlinesToOpenDocument :: WriterOptions -> [Inline] -> State WriterState Doc
-inlinesToOpenDocument o l = hcat <$> mapM (inlineToOpenDocument o) l
+inlinesToOpenDocument o l = hcat <$> toChunks o l
+
+toChunks :: WriterOptions -> [Inline] -> State WriterState [Doc]
+toChunks _ [] = return []
+toChunks o (x : xs)
+  | isChunkable x = do
+        contents <- (inTextStyle . hcat) =<<
+                     mapM (inlineToOpenDocument o) (x:ys)
+        rest <- toChunks o zs
+        return (contents : rest)
+  | otherwise     = do
+        contents <- inlineToOpenDocument o x
+        rest <- toChunks o xs
+        return (contents : rest)
+  where (ys, zs) = span isChunkable xs
+
+isChunkable :: Inline -> Bool
+isChunkable (Str _) = True
+isChunkable Space   = True
+isChunkable SoftBreak = True
+isChunkable _ = False
 
 -- | Convert an inline element to OpenDocument.
 inlineToOpenDocument :: WriterOptions -> Inline -> State WriterState Doc
 inlineToOpenDocument o ils
   = case ils of
-    Space         -> inTextStyle space
+    Space         -> return space
     SoftBreak
      | writerWrapText o == WrapPreserve
-                  -> inTextStyle (preformatted "\n")
-     | otherwise  -> inTextStyle space
+                  -> return $ preformatted "\n"
+     | otherwise  -> return $ space
     Span _ xs     -> inlinesToOpenDocument o xs
     LineBreak     -> return $ selfClosingTag "text:line-break" []
-    Str         s -> inTextStyle $ handleSpaces $ escapeStringForXML s
+    Str         s -> return $ handleSpaces $ escapeStringForXML s
     Emph        l -> withTextStyle Italic $ inlinesToOpenDocument o l
     Strong      l -> withTextStyle Bold   $ inlinesToOpenDocument o l
     Strikeout   l -> withTextStyle Strike $ inlinesToOpenDocument o l
@@ -390,7 +431,7 @@ inlineToOpenDocument o ils
     Subscript   l -> withTextStyle Sub    $ inlinesToOpenDocument o l
     SmallCaps   l -> withTextStyle SmallC $ inlinesToOpenDocument o l
     Quoted    t l -> inQuotes t <$> inlinesToOpenDocument o l
-    Code      _ s -> withTextStyle Pre $ inTextStyle $ preformatted s
+    Code      _ s -> inlinedCode $ preformatted s
     Math      t s -> inlinesToOpenDocument o (texMathToInlines t s)
     Cite      _ l -> inlinesToOpenDocument o l
     RawInline f s -> if f == Format "opendocument"
@@ -401,6 +442,7 @@ inlineToOpenDocument o ils
     Note        l  -> mkNote l
     where
       preformatted s = handleSpaces $ escapeStringForXML s
+      inlinedCode s = return $ inTags False "text:span" [("text:style-name", "Source_Text")] s
       mkLink   s t = inTags False "text:a" [ ("xlink:type" , "simple")
                                            , ("xlink:href" , s       )
                                            , ("office:name", t       )
@@ -496,15 +538,14 @@ tableStyle num wcs =
         columnStyles   = map colStyle wcs
     in  table $$ vcat columnStyles $$ cellStyle
 
-paraStyle :: String -> [(String,String)] -> State WriterState Int
-paraStyle parent attrs = do
+paraStyle :: [(String,String)] -> State WriterState Int
+paraStyle attrs = do
   pn <- (+)   1 . length       <$> gets stParaStyles
   i  <- (*) 0.5 . fromIntegral <$> gets stIndentPara :: State WriterState Double
   b  <- gets stInDefinition
   t  <- gets stTight
   let styleAttr = [ ("style:name"             , "P" ++ show pn)
-                  , ("style:family"           , "paragraph"   )
-                  , ("style:parent-style-name", parent        )]
+                  , ("style:family"           , "paragraph"   )]
       indentVal = flip (++) "in" . show $ if b then (max 0.5 i) else i
       tight     = if t then [ ("fo:margin-top"          , "0in"    )
                             , ("fo:margin-bottom"       , "0in"    )]
@@ -522,7 +563,9 @@ paraStyle parent attrs = do
   return pn
 
 paraListStyle :: Int -> State WriterState Int
-paraListStyle l = paraStyle "Text_20_body" [("style:list-style-name", "L" ++ show l )]
+paraListStyle l = paraStyle
+  [("style:parent-style-name","Text_20_body")
+  ,("style:list-style-name", "L" ++ show l )]
 
 paraTableStyles :: String -> Int -> [Alignment] -> [(String, Doc)]
 paraTableStyles _ _ [] = []
