@@ -39,8 +39,8 @@ import Text.Pandoc.Shared
 import Text.Pandoc.Writers.Shared
 import Text.Pandoc.Pretty
 import Text.Pandoc.Templates (renderTemplate')
-import Data.Char ( toLower )
-import Data.List ( intersect, intersperse, partition, transpose )
+import Data.Char ( isAlphaNum, toLower )
+import Data.List ( isPrefixOf, intersect, intersperse, partition, transpose )
 import Control.Monad.State
 
 data WriterState =
@@ -78,9 +78,9 @@ pandocToOrg (Pandoc meta blocks) = do
   let context = defField "body" main
               $ defField "math" hasMath
               $ metadata
-  if writerStandalone opts
-     then return $ renderTemplate' (writerTemplate opts) context
-     else return main
+  case writerTemplate opts of
+       Nothing  -> return main
+       Just tpl -> return $ renderTemplate' tpl context
 
 -- | Return Org representation of notes.
 notesToOrg :: [[Block]] -> State WriterState Doc
@@ -92,7 +92,7 @@ notesToOrg notes =
 noteToOrg :: Int -> [Block] -> State WriterState Doc
 noteToOrg num note = do
   contents <- blockListToOrg note
-  let marker = "[" ++ show num ++ "] "
+  let marker = "[fn:" ++ show num ++ "] "
   return $ hang (length marker) (text marker) contents
 
 -- | Escape special characters for Org.
@@ -158,13 +158,23 @@ blockToOrg (Plain inlines) = inlineListToOrg inlines
 blockToOrg (Para [Image attr txt (src,'f':'i':'g':':':tit)]) = do
   capt <- if null txt
              then return empty
-             else (\c -> "#+CAPTION: " <> c <> blankline) `fmap`
-                    inlineListToOrg txt
+             else ("#+CAPTION: " <>) `fmap` inlineListToOrg txt
   img <- inlineToOrg (Image attr txt (src,tit))
-  return $ capt <> img
+  return $ capt $$ img $$ blankline
 blockToOrg (Para inlines) = do
   contents <- inlineListToOrg inlines
   return $ contents <> blankline
+blockToOrg (LineBlock lns) = do
+  let splitStanza [] = []
+      splitStanza xs = case break (== mempty) xs of
+        (l, [])  -> l : []
+        (l, _:r) -> l : splitStanza r
+  let joinWithLinefeeds  = nowrap . mconcat . intersperse cr
+  let joinWithBlankLines = mconcat . intersperse blankline
+  let prettyfyStanza ls  = joinWithLinefeeds <$> mapM inlineListToOrg ls
+  contents <- joinWithBlankLines <$> mapM prettyfyStanza (splitStanza lns)
+  return $ blankline $$ "#+BEGIN_VERSE" $$
+           nest 2 contents $$ "#+END_VERSE" <> blankline
 blockToOrg (RawBlock "html" str) =
   return $ blankline $$ "#+BEGIN_HTML" $$
            nest 2 (text str) $$ "#+END_HTML" $$ blankline
@@ -182,11 +192,7 @@ blockToOrg (Header level attr inlines) = do
 blockToOrg (CodeBlock (_,classes,_) str) = do
   opts <- stOptions <$> get
   let tabstop = writerTabStop opts
-  let at = classes `intersect` ["asymptote", "C", "clojure", "css", "ditaa",
-                    "dot", "emacs-lisp", "gnuplot", "haskell", "js", "latex",
-                    "ledger", "lisp", "matlab", "mscgen", "ocaml", "octave",
-                    "oz", "perl", "plantuml", "python", "R", "ruby", "sass",
-                    "scheme", "screen", "sh", "sql", "sqlite"]
+  let at = map pandocLangToOrg classes `intersect` orgLangIdentifiers
   let (beg, end) = case at of
                       []    -> ("#+BEGIN_EXAMPLE", "#+END_EXAMPLE")
                       (x:_) -> ("#+BEGIN_SRC " ++ x, "#+END_SRC")
@@ -355,16 +361,56 @@ inlineToOrg (Link _ txt (src, _)) = do
   case txt of
         [Str x] | escapeURI x == src ->  -- autolink
              do modify $ \s -> s{ stLinks = True }
-                return $ "[[" <> text x <> "]]"
+                return $ "[[" <> text (orgPath x) <> "]]"
         _ -> do contents <- inlineListToOrg txt
                 modify $ \s -> s{ stLinks = True }
-                return $ "[[" <> text src <> "][" <> contents <> "]]"
+                return $ "[[" <> text (orgPath src) <> "][" <> contents <> "]]"
 inlineToOrg (Image _ _ (source, _)) = do
   modify $ \s -> s{ stImages = True }
-  return $ "[[" <> text source <> "]]"
+  return $ "[[" <> text (orgPath source) <> "]]"
 inlineToOrg (Note contents) = do
   -- add to notes in state
   notes <- get >>= (return . stNotes)
   modify $ \st -> st { stNotes = contents:notes }
   let ref = show $ (length notes) + 1
-  return $ " [" <> text ref <> "]"
+  return $ "[fn:" <> text ref <> "]"
+
+orgPath :: String -> String
+orgPath src =
+  case src of
+    []                 -> mempty         -- wiki link
+    ('#':xs)           -> xs             -- internal link
+    _ | isUrl src      -> src
+    _ | isFilePath src -> src
+    _                  -> "file:" <> src
+ where
+   isFilePath :: String -> Bool
+   isFilePath cs = any (`isPrefixOf` cs) ["/", "./", "../", "file:"]
+
+   isUrl :: String -> Bool
+   isUrl cs =
+     let (scheme, path) = break (== ':') cs
+     in all (\c -> isAlphaNum c || c `elem` (".-"::String)) scheme
+          && not (null path)
+
+-- | Translate from pandoc's programming language identifiers to those used by
+-- org-mode.
+pandocLangToOrg :: String -> String
+pandocLangToOrg cs =
+  case cs of
+    "c"          -> "C"
+    "cpp"        -> "C++"
+    "commonlisp" -> "lisp"
+    "r"          -> "R"
+    "bash"       -> "sh"
+    _            -> cs
+
+-- | List of language identifiers recognized by org-mode.
+orgLangIdentifiers :: [String]
+orgLangIdentifiers =
+  [ "asymptote", "awk", "C", "C++", "clojure", "css", "d", "ditaa", "dot"
+  , "calc", "emacs-lisp", "fortran", "gnuplot", "haskell", "java", "js"
+  , "latex", "ledger", "lisp", "lilypond", "matlab", "mscgen", "ocaml"
+  , "octave", "org", "oz", "perl", "plantuml", "processing", "python", "R"
+  , "ruby", "sass", "scheme", "screen", "sed", "sh", "sql", "sqlite"
+  ]
