@@ -107,11 +107,11 @@ writeRTF options (Pandoc meta@(Meta metamap) blocks) =
                           (tableOfContents $ filter isTOCHeader blocks)
                     else id)
               $ metadata
-  in  if writerStandalone options
-         then renderTemplate' (writerTemplate options) context
-         else case reverse body of
-                ('\n':_) -> body
-                _        -> body ++ "\n"
+  in  case writerTemplate options of
+           Just tpl -> renderTemplate' tpl context
+           Nothing  -> case reverse body of
+                            ('\n':_) -> body
+                            _        -> body ++ "\n"
 
 -- | Construct table of contents from list of header blocks.
 tableOfContents :: [Block] -> String
@@ -233,6 +233,8 @@ blockToRTF indent alignment (Plain lst) =
   rtfCompact indent 0 alignment $ inlineListToRTF lst
 blockToRTF indent alignment (Para lst) =
   rtfPar indent 0 alignment $ inlineListToRTF lst
+blockToRTF indent alignment (LineBlock lns) =
+  blockToRTF indent alignment $ linesToPara lns
 blockToRTF indent alignment (BlockQuote lst) =
   concatMap (blockToRTF (indent + indentIncrement) alignment) lst
 blockToRTF indent _ (CodeBlock _ str) =
