@@ -93,9 +93,9 @@ pandocToTexinfo options (Pandoc meta blocks) = do
               $ defField "superscript" (stSuperscript st)
               $ defField "strikeout" (stStrikeout st)
               $ metadata
-  if writerStandalone options
-     then return $ renderTemplate' (writerTemplate options) context
-     else return body
+  case writerTemplate options of
+       Nothing  -> return body
+       Just tpl -> return $ renderTemplate' tpl context
 
 -- | Escape things as needed for Texinfo.
 stringToTexinfo :: String -> String
@@ -145,6 +145,9 @@ blockToTexinfo (Para [Image attr txt (src,'f':'i':'g':':':tit)]) = do
 blockToTexinfo (Para lst) =
   inlineListToTexinfo lst    -- this is handled differently from Plain in blockListToTexinfo
 
+blockToTexinfo (LineBlock lns) =
+  blockToTexinfo $ linesToPara lns
+
 blockToTexinfo (BlockQuote lst) = do
   contents <- blockListToTexinfo lst
   return $ text "@quotation" $$
