@@ -26,11 +26,11 @@ noNorm :: Pandoc -> NoNormPandoc
 noNorm = NoNormPandoc
 
 instance ToString NoNormPandoc where
-  toString d = writeNative def{ writerStandalone = s } $ toPandoc d
+  toString d = writeNative def{ writerTemplate = s } $ toPandoc d
    where s = case d of
                   NoNormPandoc (Pandoc (Meta m) _)
-                    | M.null m  -> False
-                    | otherwise -> True
+                    | M.null m  -> Nothing
+                    | otherwise -> Just "" -- need this to get meta output
 
 instance ToPandoc NoNormPandoc where
   toPandoc = unNoNorm
@@ -181,6 +181,10 @@ tests = [ testGroup "inlines"
             "headers already having auto identifiers"
             "docx/already_auto_ident.docx"
             "docx/already_auto_ident.native"
+          , testCompare
+            "nested anchor spans in header"
+            "docx/nested_anchors_in_header.docx"
+            "docx/nested_anchors_in_header.native"
           , testCompare
             "single numbered item not made into list"
             "docx/numbered_header.docx"
@@ -237,6 +241,10 @@ tests = [ testGroup "inlines"
             "tables with lists in cells"
             "docx/table_with_list_cell.docx"
             "docx/table_with_list_cell.native"
+          , testCompare
+            "tables with one row"
+            "docx/table_one_row.docx"
+            "docx/table_one_row.native"
           , testCompare
             "code block"
             "docx/codeblock.docx"
