@@ -4,12 +4,10 @@ module Tests.Readers.Markdown (tests) where
 import Text.Pandoc.Definition
 import Test.Framework
 import Tests.Helpers
-import Tests.Arbitrary()
+import Text.Pandoc.Arbitrary()
 import Text.Pandoc.Builder
 import qualified Data.Set as Set
--- import Text.Pandoc.Shared ( normalize )
 import Text.Pandoc
-import Text.Pandoc.Error
 
 markdown :: String -> Pandoc
 markdown = handleError . readMarkdown def
@@ -36,7 +34,10 @@ testBareLink (inp, ils) =
        inp (inp, doc $ para ils)
 
 autolink :: String -> Inlines
-autolink s = link s "" (str s)
+autolink = autolinkWith nullAttr
+
+autolinkWith :: Attr -> String -> Inlines
+autolinkWith attr s = linkWith attr s "" (str s)
 
 bareLinkTests :: [(String, Inlines)]
 bareLinkTests =
@@ -148,7 +149,8 @@ tests = [ testGroup "inline code"
                 (codeWith ("",["javascript"],[]) "document.write(\"Hello\");")
           , "with attribute space" =:
             "`*` {.haskell .special x=\"7\"}"
-            =?> para (codeWith ("",["haskell","special"],[("x","7")]) "*")
+            =?> para (code "*" <> space <> str "{.haskell" <> space <>
+                      str ".special" <> space <> str "x=\"7\"}")
           ]
         , testGroup "emph and strong"
           [ "two strongs in emph" =:
@@ -185,6 +187,11 @@ tests = [ testGroup "inline code"
             "<\n\na>" =?>
             para (text "<") <> para (text "a>")
           ]
+        , testGroup "raw email addresses"
+          [ test markdownGH "issue 2940" $
+            "**@user**" =?>
+            para (strong (text "@user"))
+          ]
         , testGroup "emoji"
           [ test markdownGH "emoji symbols" $
             ":smile: and :+1:" =?> para (text "😄 and 👍")
@@ -214,6 +221,12 @@ tests = [ testGroup "inline code"
           , "a partial URL (#2277)" =:
             "<www.boe.es/buscar/act.php?id=BOE-A-1996-8930#a66>" =?>
             para (text "<www.boe.es/buscar/act.php?id=BOE-A-1996-8930#a66>")
+          , "with some attributes" =:
+            "<http://foo.bar>{#i .j .z k=v}" =?>
+            para (autolinkWith ("i", ["j", "z"], [("k", "v")]) "http://foo.bar")
+          , "with some attributes and spaces" =:
+            "<http://foo.bar> {#i .j .z k=v}" =?>
+            para (autolink "http://foo.bar" <> space <> text "{#i .j .z k=v}")
           ]
         , testGroup "links"
           [ "no autolink inside link" =:
