@@ -7,7 +7,6 @@ import Tests.Helpers
 import Text.Pandoc.Builder
 import Text.Pandoc
 import Data.List (intersperse)
-import Text.Pandoc.Error
 
 org :: String -> Pandoc
 org = handleError . readOrg def
@@ -48,6 +47,10 @@ tests =
           "/*strength*/" =?>
           para (emph . strong $ "strength")
 
+      , "Emphasized Strong preceded by space" =:
+          " */super/*" =?>
+          para (strong . emph $ "super")
+
       , "Strikeout" =:
           "+Kill Bill+" =?>
           para (strikeout . spcSep $ [ "Kill", "Bill" ])
@@ -189,13 +192,29 @@ tests =
           "=is_subst = True=" =?>
           para (code "is_subst = True")
 
-      , "Image" =:
-          "[[./sunset.jpg]]" =?>
-          (para $ image "./sunset.jpg" "" "")
+      , testGroup "Images"
+        [ "Image" =:
+            "[[./sunset.jpg]]" =?>
+            (para $ image "./sunset.jpg" "" "")
 
-      , "Image with explicit file: prefix" =:
-          "[[file:sunrise.jpg]]" =?>
-          (para $ image "sunrise.jpg" "" "")
+        , "Image with explicit file: prefix" =:
+            "[[file:sunrise.jpg]]" =?>
+            (para $ image "sunrise.jpg" "" "")
+
+        , "Multiple images within a paragraph" =:
+            unlines [ "[[file:sunrise.jpg]]"
+                    , "[[file:sunset.jpg]]"
+                    ] =?>
+            (para $ (image "sunrise.jpg" "" "")
+                 <> softbreak
+                 <> (image "sunset.jpg" "" ""))
+
+        , "Image with html attributes" =:
+            unlines [ "#+ATTR_HTML: :width 50%"
+                    , "[[file:guinea-pig.gif]]"
+                    ] =?>
+            (para $ imageWith ("", [], [("width", "50%")]) "guinea-pig.gif" "" "")
+        ]
 
       , "Explicit link" =:
           "[[http://zeitlens.com/][pseudo-random /nonsense/]]" =?>
@@ -219,12 +238,12 @@ tests =
           (para $ link "" "" "New Link")
 
       , "Image link" =:
-          "[[sunset.png][dusk.svg]]" =?>
+          "[[sunset.png][file:dusk.svg]]" =?>
           (para $ link "sunset.png" "" (image "dusk.svg" "" ""))
 
       , "Image link with non-image target" =:
-          "[[http://example.com][logo.png]]" =?>
-          (para $ link "http://example.com" "" (image "logo.png" "" ""))
+          "[[http://example.com][./logo.png]]" =?>
+          (para $ link "http://example.com" "" (image "./logo.png" "" ""))
 
       , "Plain link" =:
           "Posts on http://zeitlens.com/ can be funny at times." =?>
@@ -315,6 +334,18 @@ tests =
                        }
         in (para $ cite [citation] "cite:pandoc")
 
+      , "Org-ref simple citation succeeded by comma" =:
+        "cite:pandoc," =?>
+        let citation = Citation
+                       { citationId = "pandoc"
+                       , citationPrefix = mempty
+                       , citationSuffix = mempty
+                       , citationMode = AuthorInText
+                       , citationNoteNum = 0
+                       , citationHash = 0
+                       }
+        in (para $ cite [citation] "cite:pandoc" <> str ",")
+
       , "Org-ref simple citep citation" =:
         "citep:pandoc" =?>
         let citation = Citation
@@ -467,7 +498,14 @@ tests =
       , "Author" =:
         "#+author: Albert /Emacs-Fanboy/ Krewinkel" =?>
         let author = toList . spcSep $ [ "Albert", emph "Emacs-Fanboy", "Krewinkel" ]
-            meta = setMeta "author" (MetaInlines author) $ nullMeta
+            meta = setMeta "author" (MetaList [MetaInlines author]) $ nullMeta
+        in Pandoc meta mempty
+
+      , "Multiple authors" =:
+        "#+author: James Dewey Watson, Francis Harry Compton Crick " =?>
+        let watson = MetaInlines $ toList "James Dewey Watson"
+            crick = MetaInlines $ toList "Francis Harry Compton Crick"
+            meta = setMeta "author" (MetaList [watson, crick]) $ nullMeta
         in Pandoc meta mempty
 
       , "Date" =:
@@ -478,8 +516,8 @@ tests =
 
       , "Description" =:
         "#+DESCRIPTION: Explanatory text" =?>
-        let description = toList . spcSep $ [ "Explanatory", "text" ]
-            meta = setMeta "description" (MetaInlines description) $ nullMeta
+        let description = "Explanatory text"
+            meta = setMeta "description" (MetaString description) $ nullMeta
         in Pandoc meta mempty
 
       , "Properties drawer" =:
@@ -489,6 +527,38 @@ tests =
                   ] =?>
           (mempty::Blocks)
 
+      , "LaTeX_headers options are translated to header-includes" =:
+          "#+LaTeX_header: \\usepackage{tikz}" =?>
+          let latexInlines = rawInline "latex" "\\usepackage{tikz}"
+              inclList = MetaList [MetaInlines (toList latexInlines)]
+              meta = setMeta "header-includes" inclList nullMeta
+          in Pandoc meta mempty
+
+      , "LaTeX_class option is translated to documentclass" =:
+          "#+LATEX_CLASS: article" =?>
+          let meta = setMeta "documentclass" (MetaString "article") nullMeta
+          in Pandoc meta mempty
+
+      , "LaTeX_class_options is translated to classoption" =:
+          "#+LATEX_CLASS_OPTIONS: [a4paper]" =?>
+          let meta = setMeta "classoption" (MetaString "a4paper") nullMeta
+          in Pandoc meta mempty
+
+      , "LaTeX_class_options is translated to classoption" =:
+          "#+html_head: <meta/>" =?>
+          let html = rawInline "html" "<meta/>"
+              inclList = MetaList [MetaInlines (toList html)]
+              meta = setMeta "header-includes" inclList nullMeta
+          in Pandoc meta mempty
+
+      , "later meta definitions take precedence" =:
+          unlines [ "#+AUTHOR: this will not be used"
+                  , "#+author: Max"
+                  ] =?>
+          let author = MetaInlines [Str "Max"]
+              meta = setMeta "author" (MetaList [author]) $ nullMeta
+          in Pandoc meta mempty
+
       , "Logbook drawer" =:
           unlines [ "  :LogBook:"
                   , "  - State \"DONE\"       from \"TODO\"       [2014-03-03 Mon 11:00]"
@@ -563,69 +633,97 @@ tests =
                   ] =?>
           (para (link "http://example.com/foo" "" "bar"))
 
-      , "Export option: Disable simple sub/superscript syntax" =:
-          unlines [ "#+OPTIONS: ^:nil"
-                  , "a^b"
-                  ] =?>
-          para "a^b"
 
-      , "Export option: directly select drawers to be exported" =:
-          unlines [ "#+OPTIONS: d:(\"IMPORTANT\")"
-                  , ":IMPORTANT:"
-                  , "23"
-                  , ":END:"
-                  , ":BORING:"
-                  , "very boring"
-                  , ":END:"
-                  ] =?>
-          divWith (mempty, ["IMPORTANT", "drawer"], mempty) (para "23")
+      , testGroup "export options"
 
-      , "Export option: exclude drawers from being exported" =:
-          unlines [ "#+OPTIONS: d:(not \"BORING\")"
-                  , ":IMPORTANT:"
-                  , "5"
-                  , ":END:"
-                  , ":BORING:"
-                  , "very boring"
-                  , ":END:"
-                  ] =?>
-          divWith (mempty, ["IMPORTANT", "drawer"], mempty) (para "5")
+          [ "disable simple sub/superscript syntax" =:
+              unlines [ "#+OPTIONS: ^:nil"
+                      , "a^b"
+                      ] =?>
+              para "a^b"
 
-      , "Export option: don't include archive trees" =:
-          unlines [ "#+OPTIONS: arch:nil"
-                  , "* old  :ARCHIVE:"
-                  ] =?>
-          (mempty ::Blocks)
+          , "directly select drawers to be exported" =:
+              unlines [ "#+OPTIONS: d:(\"IMPORTANT\")"
+                      , ":IMPORTANT:"
+                      , "23"
+                      , ":END:"
+                      , ":BORING:"
+                      , "very boring"
+                      , ":END:"
+                      ] =?>
+              divWith (mempty, ["IMPORTANT", "drawer"], mempty) (para "23")
 
-      , "Export option: include complete archive trees" =:
-          unlines [ "#+OPTIONS: arch:t"
-                  , "* old  :ARCHIVE:"
-                  , "  boring"
-                  ] =?>
-          let tagSpan t = spanWith ("", ["tag"], [("data-tag-name", t)]) mempty
-          in mconcat [ headerWith ("old", [], mempty) 1 ("old" <> tagSpan "ARCHIVE")
-                     , para "boring"
-                     ]
+          , "exclude drawers from being exported" =:
+              unlines [ "#+OPTIONS: d:(not \"BORING\")"
+                      , ":IMPORTANT:"
+                      , "5"
+                      , ":END:"
+                      , ":BORING:"
+                      , "very boring"
+                      , ":END:"
+                      ] =?>
+              divWith (mempty, ["IMPORTANT", "drawer"], mempty) (para "5")
 
-      , "Export option: include archive tree header only" =:
-          unlines [ "#+OPTIONS: arch:headline"
-                  , "* old  :ARCHIVE:"
-                  , "  boring"
-                  ] =?>
-          let tagSpan t = spanWith ("", ["tag"], [("data-tag-name", t)]) mempty
-          in headerWith ("old", [], mempty) 1 ("old" <> tagSpan "ARCHIVE")
+          , "don't include archive trees" =:
+              unlines [ "#+OPTIONS: arch:nil"
+                      , "* old  :ARCHIVE:"
+                      ] =?>
+              (mempty ::Blocks)
 
-      , "Export option: limit headline depth" =:
-          unlines [ "#+OPTIONS: H:2"
-                  , "* section"
-                  , "** subsection"
-                  , "*** list item 1"
-                  , "*** list item 2"
-                  ] =?>
-          mconcat [ headerWith ("section", [], [])    1 "section"
-                  , headerWith ("subsection", [], []) 2 "subsection"
-                  , orderedList [ para "list item 1", para "list item 2" ]
-                  ]
+          , "include complete archive trees" =:
+              unlines [ "#+OPTIONS: arch:t"
+                      , "* old  :ARCHIVE:"
+                      , "  boring"
+                      ] =?>
+              let tagSpan t = spanWith ("", ["tag"], [("data-tag-name", t)]) mempty
+              in mconcat [ headerWith ("old", [], mempty) 1 ("old" <> tagSpan "ARCHIVE")
+                         , para "boring"
+                         ]
+
+          , "include archive tree header only" =:
+              unlines [ "#+OPTIONS: arch:headline"
+                      , "* old  :ARCHIVE:"
+                      , "  boring"
+                      ] =?>
+              let tagSpan t = spanWith ("", ["tag"], [("data-tag-name", t)]) mempty
+              in headerWith ("old", [], mempty) 1 ("old" <> tagSpan "ARCHIVE")
+
+          , "limit headline depth" =:
+              unlines [ "#+OPTIONS: H:2"
+                      , "* section"
+                      , "** subsection"
+                      , "*** list item 1"
+                      , "*** list item 2"
+                      ] =?>
+              mconcat [ headerWith ("section", [], [])    1 "section"
+                      , headerWith ("subsection", [], []) 2 "subsection"
+                      , orderedList [ para "list item 1", para "list item 2" ]
+                      ]
+
+          , "disable author export" =:
+              unlines [ "#+OPTIONS: author:nil"
+                      , "#+AUTHOR: ShyGuy"
+                      ] =?>
+              Pandoc nullMeta mempty
+
+          , "disable creator export" =:
+              unlines [ "#+OPTIONS: creator:nil"
+                      , "#+creator: The Architect"
+                      ] =?>
+              Pandoc nullMeta mempty
+
+          , "disable email export" =:
+              unlines [ "#+OPTIONS: email:nil"
+                      , "#+email: no-mail-please@example.com"
+                      ] =?>
+              Pandoc nullMeta mempty
+
+          , "disable inclusion of todo keywords" =:
+              unlines [ "#+OPTIONS: todo:nil"
+                      , "** DONE todo export"
+                      ] =?>
+              headerWith ("todo-export", [], []) 2 "todo export"
+          ]
       ]
 
   , testGroup "Basic Blocks" $
@@ -633,130 +731,179 @@ tests =
           "Paragraph\n" =?>
           para "Paragraph"
 
-      , "First Level Header" =:
-          "* Headline\n" =?>
-          headerWith ("headline", [], []) 1 "Headline"
+      , testGroup "headers" $
+        [ "First Level Header" =:
+            "* Headline\n" =?>
+            headerWith ("headline", [], []) 1 "Headline"
 
-      , "Third Level Header" =:
-          "*** Third Level Headline\n" =?>
-          headerWith ("third-level-headline", [], [])
-                     3
-                     ("Third" <> space <> "Level" <> space <> "Headline")
+        , "Third Level Header" =:
+            "*** Third Level Headline\n" =?>
+            headerWith ("third-level-headline", [], [])
+                       3
+                       ("Third" <> space <> "Level" <> space <> "Headline")
 
-      , "Compact Headers with Paragraph" =:
-          unlines [ "* First Level"
-                  , "** Second Level"
-                  , "   Text"
-                  ] =?>
-          mconcat [ headerWith ("first-level", [], [])
-                               1
-                               ("First" <> space <> "Level")
-                  , headerWith ("second-level", [], [])
-                               2
-                               ("Second" <> space <> "Level")
-                  , para "Text"
-                  ]
+        , "Compact Headers with Paragraph" =:
+            unlines [ "* First Level"
+                    , "** Second Level"
+                    , "   Text"
+                    ] =?>
+            mconcat [ headerWith ("first-level", [], [])
+                                 1
+                                 ("First" <> space <> "Level")
+                    , headerWith ("second-level", [], [])
+                                 2
+                                 ("Second" <> space <> "Level")
+                    , para "Text"
+                    ]
 
-      , "Separated Headers with Paragraph" =:
-          unlines [ "* First Level"
-                  , ""
-                  , "** Second Level"
-                  , ""
-                  , "   Text"
-                  ] =?>
-          mconcat [ headerWith ("first-level", [], [])
-                               1
-                               ("First" <> space <> "Level")
-                  , headerWith ("second-level", [], [])
-                               2
-                               ("Second" <> space <> "Level")
-                  , para "Text"
-                  ]
+        , "Separated Headers with Paragraph" =:
+            unlines [ "* First Level"
+                    , ""
+                    , "** Second Level"
+                    , ""
+                    , "   Text"
+                    ] =?>
+            mconcat [ headerWith ("first-level", [], [])
+                                 1
+                                 ("First" <> space <> "Level")
+                    , headerWith ("second-level", [], [])
+                                 2
+                                 ("Second" <> space <> "Level")
+                    , para "Text"
+                    ]
 
-      , "Headers not preceded by a blank line" =:
-          unlines [ "** eat dinner"
-                  , "Spaghetti and meatballs tonight."
-                  , "** walk dog"
-                  ] =?>
-          mconcat [ headerWith ("eat-dinner", [], [])
-                               2
-                               ("eat" <> space <> "dinner")
-                  , para $ spcSep [ "Spaghetti", "and", "meatballs", "tonight." ]
-                  , headerWith ("walk-dog", [], [])
-                               2
-                               ("walk" <> space <> "dog")
-                  ]
+        , "Headers not preceded by a blank line" =:
+            unlines [ "** eat dinner"
+                    , "Spaghetti and meatballs tonight."
+                    , "** walk dog"
+                    ] =?>
+            mconcat [ headerWith ("eat-dinner", [], [])
+                                 2
+                                 ("eat" <> space <> "dinner")
+                    , para $ spcSep [ "Spaghetti", "and", "meatballs", "tonight." ]
+                    , headerWith ("walk-dog", [], [])
+                                 2
+                                 ("walk" <> space <> "dog")
+                    ]
 
-      , "Tagged headers" =:
-          unlines [ "* Personal       :PERSONAL:"
-                  , "** Call Mom      :@PHONE:"
-                  , "** Call John     :@PHONE:JOHN: "
-                  ] =?>
-          let tagSpan t = spanWith ("", ["tag"], [("data-tag-name", t)]) mempty
-          in mconcat [ headerWith ("personal", [], [])
-                                  1
-                                  ("Personal" <> tagSpan "PERSONAL")
-                     , headerWith ("call-mom", [], [])
-                                  2
-                                  ("Call Mom" <> tagSpan "@PHONE")
-                     , headerWith ("call-john", [], [])
-                                  2
-                                  ("Call John" <> tagSpan "@PHONE" <> tagSpan "JOHN")
-                     ]
+        , testGroup "Todo keywords"
+          [ "Header with known todo keyword" =:
+              "* TODO header" =?>
+              let todoSpan = spanWith ("", ["todo", "TODO"], []) "TODO"
+              in headerWith ("header", [], []) 1 (todoSpan <> space <> "header")
 
-      , "Untagged header containing colons" =:
-          "* This: is not: tagged" =?>
-          headerWith ("this-is-not-tagged", [], []) 1 "This: is not: tagged"
+          , "Header marked as done" =:
+              "* DONE header" =?>
+              let todoSpan = spanWith ("", ["done", "DONE"], []) "DONE"
+              in headerWith ("header", [], []) 1 (todoSpan <> space <> "header")
 
-      , "Header starting with strokeout text" =:
-          unlines [ "foo"
-                  , ""
-                  , "* +thing+ other thing"
-                  ] =?>
-          mconcat [ para "foo"
-                  , headerWith ("thing-other-thing", [], [])
-                               1
-                               ((strikeout "thing") <> " other thing")
-                  ]
+          , "Header with unknown todo keyword" =:
+              "* WAITING header" =?>
+              headerWith ("waiting-header", [], []) 1 "WAITING header"
 
-      , "Comment Trees" =:
-          unlines [ "* COMMENT A comment tree"
-                  , "  Not much going on here"
-                  , "** This will be dropped"
-                  , "* Comment tree above"
-                  ] =?>
-          headerWith ("comment-tree-above", [], []) 1 "Comment tree above"
+          , "Custom todo keywords" =:
+              unlines [ "#+TODO: WAITING CANCELLED"
+                      , "* WAITING compile"
+                      , "* CANCELLED lunch"
+                      ] =?>
+              let todoSpan = spanWith ("", ["todo", "WAITING"], []) "WAITING"
+                  doneSpan = spanWith ("", ["done", "CANCELLED"], []) "CANCELLED"
+              in headerWith ("compile", [], []) 1 (todoSpan <> space <> "compile")
+              <> headerWith ("lunch", [], []) 1 (doneSpan <> space <> "lunch")
 
-      , "Nothing but a COMMENT header" =:
-          "* COMMENT Test" =?>
-          (mempty::Blocks)
+          , "Custom todo keywords with multiple done-states" =:
+              unlines [ "#+TODO: WAITING | DONE CANCELLED "
+                      , "* WAITING compile"
+                      , "* CANCELLED lunch"
+                      , "* DONE todo-feature"
+                      ] =?>
+              let waiting = spanWith ("", ["todo", "WAITING"], []) "WAITING"
+                  cancelled = spanWith ("", ["done", "CANCELLED"], []) "CANCELLED"
+                  done = spanWith ("", ["done", "DONE"], []) "DONE"
+              in headerWith ("compile", [], []) 1 (waiting <> space <> "compile")
+              <> headerWith ("lunch", [], []) 1 (cancelled <> space <> "lunch")
+              <> headerWith ("todo-feature", [], []) 1 (done <> space <> "todo-feature")
+          ]
 
-      , "Tree with :noexport:" =:
-          unlines [ "* Should be ignored :archive:noexport:old:"
-                  , "** Old stuff"
-                  , "   This is not going to be exported"
-                  ] =?>
-          (mempty::Blocks)
+        , "Tagged headers" =:
+            unlines [ "* Personal       :PERSONAL:"
+                    , "** Call Mom      :@PHONE:"
+                    , "** Call John     :@PHONE:JOHN: "
+                    ] =?>
+            let tagSpan t = spanWith ("", ["tag"], [("data-tag-name", t)]) mempty
+            in mconcat [ headerWith ("personal", [], [])
+                                    1
+                                    ("Personal" <> tagSpan "PERSONAL")
+                       , headerWith ("call-mom", [], [])
+                                    2
+                                    ("Call Mom" <> tagSpan "@PHONE")
+                       , headerWith ("call-john", [], [])
+                                    2
+                                    ("Call John" <> tagSpan "@PHONE" <> tagSpan "JOHN")
+                       ]
 
-      , "Subtree with :noexport:" =:
-          unlines [ "* Exported"
-                  , "** This isn't exported :noexport:"
-                  , "*** This neither"
-                  , "** But this is"
-                  ] =?>
-          mconcat [ headerWith ("exported", [], []) 1 "Exported"
-                  , headerWith ("but-this-is", [], []) 2 "But this is"
-                  ]
+        , "Untagged header containing colons" =:
+            "* This: is not: tagged" =?>
+            headerWith ("this-is-not-tagged", [], []) 1 "This: is not: tagged"
 
-      , "Preferences are treated as header attributes" =:
-          unlines [ "* foo"
-                  , "  :PROPERTIES:"
-                  , "  :custom_id: fubar"
-                  , "  :bar: baz"
-                  , "  :END:"
-                  ] =?>
-          headerWith ("fubar", [], [("bar", "baz")]) 1 "foo"
+        , "Header starting with strokeout text" =:
+            unlines [ "foo"
+                    , ""
+                    , "* +thing+ other thing"
+                    ] =?>
+            mconcat [ para "foo"
+                    , headerWith ("thing-other-thing", [], [])
+                                 1
+                                 ((strikeout "thing") <> " other thing")
+                    ]
 
+        , "Comment Trees" =:
+            unlines [ "* COMMENT A comment tree"
+                    , "  Not much going on here"
+                    , "** This will be dropped"
+                    , "* Comment tree above"
+                    ] =?>
+            headerWith ("comment-tree-above", [], []) 1 "Comment tree above"
+
+        , "Nothing but a COMMENT header" =:
+            "* COMMENT Test" =?>
+            (mempty::Blocks)
+
+        , "Tree with :noexport:" =:
+            unlines [ "* Should be ignored :archive:noexport:old:"
+                    , "** Old stuff"
+                    , "   This is not going to be exported"
+                    ] =?>
+            (mempty::Blocks)
+
+        , "Subtree with :noexport:" =:
+            unlines [ "* Exported"
+                    , "** This isn't exported :noexport:"
+                    , "*** This neither"
+                    , "** But this is"
+                    ] =?>
+            mconcat [ headerWith ("exported", [], []) 1 "Exported"
+                    , headerWith ("but-this-is", [], []) 2 "But this is"
+                    ]
+
+        , "Preferences are treated as header attributes" =:
+            unlines [ "* foo"
+                    , "  :PROPERTIES:"
+                    , "  :custom_id: fubar"
+                    , "  :bar: baz"
+                    , "  :END:"
+                    ] =?>
+            headerWith ("fubar", [], [("bar", "baz")]) 1 "foo"
+
+
+        , "Headers marked with a unnumbered property get a class of the same name" =:
+            unlines [ "* Not numbered"
+                    , "  :PROPERTIES:"
+                    , "  :UNNUMBERED: t"
+                    , "  :END:"
+                    ] =?>
+            headerWith ("not-numbered", ["unnumbered"], []) 1 "Not numbered"
+      ]
       , "Paragraph starting with an asterisk" =:
           "*five" =?>
           para "*five"
@@ -810,29 +957,29 @@ tests =
         [ "Figure" =:
             unlines [ "#+caption: A very courageous man."
                     , "#+name: goodguy"
-                    , "[[edward.jpg]]"
+                    , "[[file:edward.jpg]]"
                     ] =?>
             para (image "edward.jpg" "fig:goodguy" "A very courageous man.")
 
         , "Figure with no name" =:
             unlines [ "#+caption: I've been through the desert on this"
-                    , "[[horse.png]]"
+                    , "[[file:horse.png]]"
                     ] =?>
             para (image "horse.png" "fig:" "I've been through the desert on this")
 
         , "Figure with `fig:` prefix in name" =:
             unlines [ "#+caption: Used as a metapher in evolutionary biology."
                     , "#+name: fig:redqueen"
-                    , "[[the-red-queen.jpg]]"
+                    , "[[./the-red-queen.jpg]]"
                     ] =?>
-            para (image "the-red-queen.jpg" "fig:redqueen"
+            para (image "./the-red-queen.jpg" "fig:redqueen"
                         "Used as a metapher in evolutionary biology.")
 
         , "Figure with HTML attributes" =:
             unlines [ "#+CAPTION: mah brain just explodid"
                     , "#+NAME: lambdacat"
                     , "#+ATTR_HTML: :style color: blue :role button"
-                    , "[[lambdacat.jpg]]"
+                    , "[[file:lambdacat.jpg]]"
                     ] =?>
             let kv = [("style", "color: blue"), ("role", "button")]
                 name = "fig:lambdacat"
@@ -842,10 +989,16 @@ tests =
         , "Labelled figure" =:
             unlines [ "#+CAPTION: My figure"
                     , "#+LABEL: fig:myfig"
-                    , "[[blub.png]]"
+                    , "[[file:blub.png]]"
                     ] =?>
             let attr = ("fig:myfig", mempty, mempty)
             in para (imageWith attr "blub.png" "fig:" "My figure")
+
+        , "Figure with empty caption" =:
+            unlines [ "#+CAPTION:"
+                    , "[[file:guess.jpg]]"
+                    ] =?>
+            para (image "guess.jpg" "fig:" "")
         ]
 
       , "Footnote" =:
@@ -1420,14 +1573,11 @@ tests =
           mconcat
           [ para $ spcSep [ "The", "first", "lines", "of"
                           , "Goethe's", emph "Faust" <> ":"]
-          , para $ mconcat
-              [ spcSep [ "Habe", "nun,", "ach!", "Philosophie," ]
-              , linebreak
-              , spcSep [ "Juristerei", "und", "Medizin," ]
-              , linebreak
-              , spcSep [ "Und", "leider", "auch", "Theologie!" ]
-              , linebreak
-              , spcSep [ "Durchaus", "studiert,", "mit", "heißem", "Bemühn." ]
+          , lineBlock
+              [ "Habe nun, ach! Philosophie,"
+              , "Juristerei und Medizin,"
+              , "Und leider auch Theologie!"
+              , "Durchaus studiert, mit heißem Bemühn."
               ]
           ]
 
@@ -1438,7 +1588,15 @@ tests =
                   , "bar"
                   , "#+END_VERSE"
                   ] =?>
-          para ("foo" <> linebreak <> linebreak <> "bar")
+          lineBlock [ "foo", mempty, "bar" ]
+
+      , "Verse block with varying indentation" =:
+          unlines [ "#+BEGIN_VERSE"
+                  , "  hello darkness"
+                  , "my old friend"
+                  , "#+END_VERSE"
+                  ] =?>
+          lineBlock [ "\160\160hello darkness", "my old friend" ]
 
       , "Raw block LaTeX" =:
           unlines [ "#+BEGIN_LaTeX"
@@ -1447,6 +1605,18 @@ tests =
                   ] =?>
           rawBlock "latex" "The category $\\cat{Set}$ is adhesive.\n"
 
+      , "Raw LaTeX line" =:
+          "#+LATEX: \\let\\foo\\bar" =?>
+          rawBlock "latex" "\\let\\foo\\bar"
+
+      , "Raw Beamer line" =:
+          "#+beamer: \\pause" =?>
+          rawBlock "beamer" "\\pause"
+
+      , "Raw HTML line" =:
+          "#+HTML: <aside>not important</aside>" =?>
+          rawBlock "html" "<aside>not important</aside>"
+
       , "Export block HTML" =:
           unlines [ "#+BEGIN_export html"
                   , "<samp>Hello, World!</samp>"
@@ -1499,6 +1669,15 @@ tests =
                   ] =?>
           rawBlock "html" "\n<span>boring</span>\n\n"
 
+      , "Accept `ATTR_HTML` attributes for generic block" =:
+          unlines [ "#+ATTR_HTML: :title hello, world :id test :class fun code"
+                  , "#+BEGIN_TEST"
+                  , "nonsense"
+                  , "#+END_TEST"
+                  ] =?>
+          let attr = ("test", ["fun", "code", "TEST"], [("title", "hello, world")])
+          in divWith attr (para "nonsense")
+
       , "Non-letter chars in source block parameters" =:
           unlines [ "#+BEGIN_SRC C :tangle xxxx.c :city Zürich"
                   , "code body"
