@@ -6,7 +6,7 @@ import Text.Pandoc.Walk
 import Test.Framework
 import Tests.Helpers
 import Data.Char (toUpper)
-import Tests.Arbitrary()
+import Text.Pandoc.Arbitrary()
 import Data.Generics
 
 tests :: [Test]
