@@ -5,7 +5,7 @@ import Test.Framework
 import Text.Pandoc.Builder
 import Text.Pandoc
 import Tests.Helpers
-import Tests.Arbitrary()
+import Text.Pandoc.Arbitrary()
 
 html :: (ToPandoc a) => a -> String
 html = writeHtmlString def{ writerWrapText = WrapNone } . toPandoc
