@@ -4,11 +4,11 @@ import Test.Framework
 import Text.Pandoc.Builder
 import Text.Pandoc
 import Tests.Helpers
-import Tests.Arbitrary()
+import Text.Pandoc.Arbitrary()
 
 p_write_rt :: Pandoc -> Bool
 p_write_rt d =
-  read (writeNative def{ writerStandalone = True } d) == d
+  read (writeNative def{ writerTemplate = Just "" } d) == d
 
 p_write_blocks_rt :: [Block] -> Bool
 p_write_blocks_rt bs = length bs > 20 ||
